@@ -9,12 +9,13 @@ from astropy import units as u
 from astropy.modeling.convolution import Convolution
 from astropy.modeling.core import SPECIAL_OPERATORS, CompoundModel
 from astropy.nddata import support_nddata
+from astropy.utils.compat.optional_deps import HAS_SCIPY
 from astropy.utils.console import human_file_size
 from astropy.utils.exceptions import AstropyUserWarning
 
 from ._convolve import _convolveNd_c
 from .core import MAX_NORMALIZATION, Kernel, Kernel1D, Kernel2D
-from .utils import KernelSizeError, has_even_axis, raise_even_kernel_exception
+from .utils import KernelSizeError, has_even_axis
 
 # np.unique([scipy.fft.next_fast_len(i, real=True) for i in range(10000)])
 # fmt: off
@@ -54,12 +55,10 @@ def _next_fast_lengths(shape):
     Calculated directly with `scipy.fft.next_fast_len`, if available; otherwise
     looked up from list and scaled by powers of 10, if necessary.
     """
-    try:
+    if HAS_SCIPY:
         import scipy.fft
 
         return np.array([scipy.fft.next_fast_len(j) for j in shape])
-    except ImportError:
-        pass
 
     newshape = np.empty(len(np.atleast_1d(shape)), dtype=int)
     for i, j in enumerate(shape):
@@ -96,12 +95,10 @@ def _copy_input_if_needed(
                 # is no way to specify the return type or order etc. In addition
                 # ``np.nan`` is a ``float`` and there is no conversion to an
                 # ``int`` type. Therefore, a pre-fill copy is needed for non
-                # ``float`` masked arrays. ``subok=True`` is needed to retain
-                # ``np.ma.maskedarray.filled()``. ``copy=False`` allows the fill
-                # to act as the copy if type and order are already correct.
-                output = np.array(
-                    input, dtype=dtype, copy=False, order=order, subok=True
-                )
+                # ``float`` masked arrays. ``asanyarray`` is needed to retain
+                # ``np.ma.maskedarray.filled()``.
+                # A copy is made if and only if order isn't already correct.
+                output = np.asanyarray(input, dtype=dtype, order=order)
                 output = output.filled(fill_value)
             else:
                 # Since we're making a copy, we might as well use `subok=False` to save,
@@ -114,11 +111,7 @@ def _copy_input_if_needed(
                 # mask != 0 yields a bool mask for all ints/floats/bool
                 output[mask != 0] = fill_value
         else:
-            # The call below is synonymous with np.asanyarray(array, ftype=float, order='C')
-            # The advantage of `subok=True` is that it won't copy when array is an ndarray subclass.
-            # If it is and `subok=False` (default), then it will copy even if `copy=False`. This
-            # uses less memory when ndarray subclasses are passed in.
-            output = np.array(input, dtype=dtype, copy=False, order=order, subok=True)
+            output = np.asanyarray(input, dtype=dtype, order=order)
     except (TypeError, ValueError) as e:
         raise TypeError(
             "input should be a Numpy array or something convertible into a float array",
@@ -262,7 +255,7 @@ def convolve(
 
     # Make sure kernel has all odd axes
     if has_even_axis(kernel_internal):
-        raise_even_kernel_exception()
+        raise KernelSizeError("Kernel size must be odd in all axes.")
 
     # If both image array and kernel are Kernel instances
     # constrain convolution method
@@ -289,13 +282,15 @@ def convolve(
 
     # Check dimensionality
     if array_internal.ndim == 0:
-        raise Exception("cannot convolve 0-dimensional arrays")
+        raise ValueError("cannot convolve 0-dimensional arrays")
     elif array_internal.ndim > 3:
         raise NotImplementedError(
             "convolve only supports 1, 2, and 3-dimensional arrays at this time"
         )
     elif array_internal.ndim != kernel_internal.ndim:
-        raise Exception("array and kernel have differing number of dimensions.")
+        raise ValueError("array and kernel have differing number of dimensions.")
+    elif array_internal.size == 0:
+        raise ValueError("cannot convolve empty array")
 
     array_shape = np.array(array_internal.shape)
     kernel_shape = np.array(kernel_internal.shape)
@@ -346,7 +341,10 @@ def convolve(
                 raise ValueError(
                     "The kernel can't be normalized, because "
                     "its sum is close to zero. The sum of the "
-                    f"given kernel is < {1.0 / MAX_NORMALIZATION}"
+                    f"given kernel is < {1.0 / MAX_NORMALIZATION:.2f}. "
+                    "For a zero-sum kernel, set normalize_kernel=False "
+                    "or pass a custom normalization function to "
+                    "normalize_kernel."
                 )
 
     # Mark the NaN values so we can replace them later if interpolate_nan is
@@ -376,9 +374,9 @@ def convolve(
             #            [pad_width[0]:-pad_width[0]]
             # to account for when the kernel has size of 1 making pad_width = 0.
             if array_internal.ndim == 1:
-                array_to_convolve[
-                    pad_width[0] : array_shape[0] + pad_width[0]
-                ] = array_internal
+                array_to_convolve[pad_width[0] : array_shape[0] + pad_width[0]] = (
+                    array_internal
+                )
             elif array_internal.ndim == 2:
                 array_to_convolve[
                     pad_width[0] : array_shape[0] + pad_width[0],
@@ -453,13 +451,9 @@ def convolve(
         if isinstance(passed_kernel, Kernel):
             new_result._separable = new_result._separable and passed_kernel._separable
         return new_result
-    elif array_dtype.kind == "f":
+    if array_dtype.kind == "f":
         # Try to preserve the input type if it's a floating point type
-        # Avoid making another copy if possible
-        try:
-            return result.astype(array_dtype, copy=False)
-        except TypeError:
-            return result.astype(array_dtype)
+        return result.astype(array_dtype, copy=False)
     else:
         return result
 
@@ -752,8 +746,11 @@ def convolve_fft(
     if normalize_kernel is True:
         if kernel.sum() < 1.0 / MAX_NORMALIZATION:
             raise Exception(
-                "The kernel can't be normalized, because its sum is close to zero. The"
-                f" sum of the given kernel is < {1.0 / MAX_NORMALIZATION}"
+                "The kernel can't be normalized, because its sum is close "
+                "to zero. The sum of the given kernel is < "
+                f"{1.0 / MAX_NORMALIZATION:.2f}. For a zero-sum kernel, set "
+                "normalize_kernel=False or pass a custom normalization "
+                "function to normalize_kernel."
             )
         kernel_scale = kernel.sum()
         normalized_kernel = kernel / kernel_scale
@@ -944,11 +941,7 @@ def convolve_fft(
     if preserve_nan:
         rifft[arrayslices][nanmaskarray] = np.nan
 
-    if crop:
-        result = rifft[arrayslices].real
-        return result
-    else:
-        return rifft.real
+    return rifft[arrayslices].real if crop else rifft.real
 
 
 def interpolate_replace_nans(array, kernel, convolve=convolve, **kwargs):
