@@ -58,7 +58,6 @@ class FITS_record:
             The ending column in the row associated with this object.
             Used for subsetting the columns of the `FITS_rec` object.
         """
-
         self.array = input
         self.row = row
         if base:
@@ -72,36 +71,37 @@ class FITS_record:
 
     def __getitem__(self, key):
         if isinstance(key, str):
-            indx = _get_index(self.array.names, key)
+            index = _get_index(self.array.names, key)
 
-            if indx < self.start or indx > self.end - 1:
+            if index < self.start or index > self.end - 1:
                 raise KeyError(f"Key '{key}' does not exist.")
         elif isinstance(key, slice):
             return type(self)(self.array, self.row, key.start, key.stop, key.step, self)
         else:
-            indx = self._get_index(key)
+            index = self._get_index(key)
 
-            if indx > self.array._nfields - 1:
+            if index > self.array._nfields - 1:
                 raise IndexError("Index out of bounds")
 
-        return self.array.field(indx)[self.row]
+        return self.array.field(index)[self.row]
 
     def __setitem__(self, key, value):
         if isinstance(key, str):
-            indx = _get_index(self.array.names, key)
+            index = _get_index(self.array.names, key)
 
-            if indx < self.start or indx > self.end - 1:
+            if index < self.start or index > self.end - 1:
                 raise KeyError(f"Key '{key}' does not exist.")
         elif isinstance(key, slice):
-            for indx in range(slice.start, slice.stop, slice.step):
-                indx = self._get_indx(indx)
-                self.array.field(indx)[self.row] = value
+            start, stop, step = key.indices(self.array._nfields)
+            for i, val in zip(range(start, stop, step), value, strict=True):
+                self.array.field(self._get_index(i))[self.row] = val
+            return
         else:
-            indx = self._get_index(key)
-            if indx > self.array._nfields - 1:
+            index = self._get_index(key)
+            if index > self.array._nfields - 1:
                 raise IndexError("Index out of bounds")
 
-        self.array.field(indx)[self.row] = value
+        self.array.field(index)[self.row] = value
 
     def __len__(self):
         return len(range(self.start, self.end, self.step))
@@ -110,7 +110,6 @@ class FITS_record:
         """
         Display a single row.
         """
-
         outlist = []
         for idx in range(len(self)):
             outlist.append(repr(self[idx]))
@@ -120,14 +119,12 @@ class FITS_record:
         """
         Get the field data of the record.
         """
-
         return self.__getitem__(field)
 
     def setfield(self, field, value):
         """
         Set the field data of the record.
         """
-
         self.__setitem__(field, value)
 
     @lazyproperty
@@ -139,7 +136,7 @@ class FITS_record:
             base = base.base
         return bases
 
-    def _get_index(self, indx):
+    def _get_index(self, index):
         indices = np.ogrid[: self.array._nfields]
         for base in reversed(self._bases):
             if base.step < 1:
@@ -147,7 +144,7 @@ class FITS_record:
             else:
                 s = slice(base.start, base.end, base.step)
             indices = indices[s]
-        return indices[indx]
+        return indices[index]
 
 
 class FITS_rec(np.recarray):
@@ -162,20 +159,18 @@ class FITS_rec(np.recarray):
 
     _record_type = FITS_record
     _character_as_bytes = False
+    _load_variable_length_data = True
 
-    def __new__(subtype, input):
+    def __new__(cls, input):
         """
         Construct a FITS record array from a recarray.
         """
-
         # input should be a record array
         if input.dtype.subdtype is None:
-            self = np.recarray.__new__(
-                subtype, input.shape, input.dtype, buf=input.data
-            )
+            self = np.recarray.__new__(cls, input.shape, input.dtype, buf=input.data)
         else:
             self = np.recarray.__new__(
-                subtype, input.shape, input.dtype, buf=input.data, strides=input.strides
+                cls, input.shape, input.dtype, buf=input.data, strides=input.strides
             )
 
         self._init()
@@ -202,7 +197,6 @@ class FITS_rec(np.recarray):
         functionality but then add in a tuple of FITS_rec-specific
         values that get used in __setstate__.
         """
-
         reconst_func, reconst_func_args, state = super().__reduce__()
 
         # Define FITS_rec-specific attrs that get added to state
@@ -213,6 +207,7 @@ class FITS_rec(np.recarray):
             "_converted",
             "_heapoffset",
             "_heapsize",
+            "_tbsize",
             "_nfields",
             "_gap",
             "_uint",
@@ -243,6 +238,7 @@ class FITS_rec(np.recarray):
             self._converted = obj._converted
             self._heapoffset = obj._heapoffset
             self._heapsize = obj._heapsize
+            self._tbsize = obj._tbsize
             self._col_weakrefs = obj._col_weakrefs
             self._coldefs = obj._coldefs
             self._nfields = obj._nfields
@@ -256,6 +252,7 @@ class FITS_rec(np.recarray):
 
             self._heapoffset = getattr(obj, "_heapoffset", 0)
             self._heapsize = getattr(obj, "_heapsize", 0)
+            self._tbsize = getattr(obj, "_tbsize", 0)
 
             self._gap = getattr(obj, "_gap", 0)
             self._uint = getattr(obj, "_uint", False)
@@ -274,11 +271,11 @@ class FITS_rec(np.recarray):
 
     def _init(self):
         """Initializes internal attributes specific to FITS-isms."""
-
         self._nfields = 0
         self._converted = {}
         self._heapoffset = 0
         self._heapsize = 0
+        self._tbsize = 0
         self._col_weakrefs = weakref.WeakSet()
         self._coldefs = None
         self._gap = 0
@@ -317,7 +314,6 @@ class FITS_rec(np.recarray):
             `False`, copy the data from input, undefined cells will still
             be filled with zeros/blanks.
         """
-
         if not isinstance(columns, ColDefs):
             columns = ColDefs(columns)
 
@@ -340,8 +336,7 @@ class FITS_rec(np.recarray):
                     dim = arr.shape[0]
                 else:
                     dim = 0
-                if dim > nrows:
-                    nrows = dim
+                nrows = max(dim, nrows)
 
         raw_data = np.empty(columns.dtype.itemsize * nrows, dtype=np.uint8)
         raw_data.fill(ord(columns._padding_byte))
@@ -375,21 +370,16 @@ class FITS_rec(np.recarray):
             arr = column.array
 
             if arr is None:
-                array_size = 0
-            else:
-                array_size = len(arr)
+                # The input column had an empty array, so just use the fill
+                # value
+                continue
 
-            n = min(array_size, nrows)
+            n = min(len(arr), nrows)
 
             # TODO: At least *some* of this logic is mostly redundant with the
             # _convert_foo methods in this class; see if we can eliminate some
             # of that duplication.
 
-            if not n:
-                # The input column had an empty array, so just use the fill
-                # value
-                continue
-
             field = _get_recarray_field(data, idx)
             name = column.name
             fitsformat = column.format
@@ -421,7 +411,7 @@ class FITS_rec(np.recarray):
                 # TODO: Maybe this step isn't necessary at all if _scale_back
                 # will handle it?
                 inarr = np.where(inarr == np.False_, ord("F"), ord("T"))
-            elif columns[idx]._physical_values and columns[idx]._pseudo_unsigned_ints:
+            elif column._physical_values and column._pseudo_unsigned_ints:
                 # Temporary hack...
                 bzero = column.bzero
                 converted = np.zeros(field.shape, dtype=inarr.dtype)
@@ -488,9 +478,30 @@ class FITS_rec(np.recarray):
         return data
 
     def __repr__(self):
-        # Force use of the normal ndarray repr (rather than the new
-        # one added for recarray in Numpy 1.10) for backwards compat
-        return np.ndarray.__repr__(self)
+        # recarray.__repr__ hard-codes the name of the class, so we overwrite.
+        # The following is mostly a straight copy except for the name change
+        # and for treating using str to typeset integer -- the latter to fix
+        # the case where the integer columns are scaled (see gh-17583). Also,
+        # removed a branch for "if the user is playing strange game with dtypes".
+        #
+        # FIXME: recarray removes the "numpy.record" mention in the dtype repr,
+        # we could do the same in a future version
+
+        repr_dtype = self.dtype
+        # if repr_dtype.type is np.record:
+        #     repr_dtype = np.dtype((np.void, repr_dtype))
+        prefix = "FITS_rec("
+        fmt = "FITS_rec(%s,%sdtype=%s)"
+        # get data/shape string. logic taken from numeric.array_repr
+        if self.size > 0 or self.shape == (0,):
+            lst = np.array2string(
+                self, separator=", ", prefix=prefix, suffix=",", formatter=dict(int=str)
+            )
+        else:
+            # show zero-length shape unless it is (0,)
+            lst = "[], shape=%s" % (repr(self.shape),)  # noqa: UP031
+        lf = "\n" + " " * len(prefix)
+        return fmt % (lst, lf, repr_dtype)
 
     def __getattribute__(self, attr):
         # First, see if ndarray has this attr, and return it if so. Note that
@@ -530,6 +541,10 @@ class FITS_rec(np.recarray):
 
         # We got a view; change it back to our class, and add stuff
         out = out.view(type(self))
+        out._heapoffset = self._heapoffset
+        out._heapsize = self._heapsize
+        out._tbsize = self._tbsize
+        out._gap = self._gap
         out._uint = self._uint
         out._coldefs = ColDefs(self._coldefs)
         arrays = []
@@ -578,9 +593,7 @@ class FITS_rec(np.recarray):
                     self.field(idx)[key] = value[idx]
             else:
                 raise ValueError(
-                    "Input tuple or list required to have {} elements.".format(
-                        self._nfields
-                    )
+                    f"Input tuple or list required to have {self._nfields} elements."
                 )
         else:
             raise TypeError(
@@ -600,16 +613,15 @@ class FITS_rec(np.recarray):
         copy of all those attributes so that the two arrays truly do not share
         any data.
         """
-
         new = super().copy(order=order)
 
         new.__dict__ = copy.deepcopy(self.__dict__)
+        new._col_weakrefs = weakref.WeakSet()
         return new
 
     @property
     def columns(self):
         """A user-visible accessor for the coldefs."""
-
         return self._coldefs
 
     @property
@@ -661,7 +673,6 @@ class FITS_rec(np.recarray):
     @property
     def names(self):
         """List of column names."""
-
         if self.dtype.fields:
             return list(self.dtype.names)
         elif getattr(self, "_coldefs", None) is not None:
@@ -672,7 +683,6 @@ class FITS_rec(np.recarray):
     @property
     def formats(self):
         """List of column FITS formats."""
-
         if getattr(self, "_coldefs", None) is not None:
             return self._coldefs.formats
 
@@ -687,7 +697,6 @@ class FITS_rec(np.recarray):
 
         Currently for internal use only.
         """
-
         if _has_unicode_fields(self):
             total_itemsize = 0
             for field in self.dtype.fields.values():
@@ -704,20 +713,12 @@ class FITS_rec(np.recarray):
         """
         A view of a `Column`'s data as an array.
         """
-
         # NOTE: The *column* index may not be the same as the field index in
         # the recarray, if the column is a phantom column
         column = self.columns[key]
         name = column.name
         format = column.format
 
-        if format.dtype.itemsize == 0:
-            warnings.warn(
-                "Field {!r} has a repeat count of 0 in its format code, "
-                "indicating an empty field.".format(key)
-            )
-            return np.array([], dtype=format.dtype)
-
         # If field's base is a FITS_rec, we can run into trouble because it
         # contains a reference to the ._coldefs object of the original data;
         # this can lead to a circular reference; see ticket #49
@@ -733,7 +734,7 @@ class FITS_rec(np.recarray):
             recformat = format.recformat
             # TODO: If we're now passing the column to these subroutines, do we
             # really need to pass them the recformat?
-            if isinstance(recformat, _FormatP):
+            if isinstance(recformat, _FormatP) and self._load_variable_length_data:
                 # for P format
                 converted = self._convert_p(column, field, recformat)
             else:
@@ -758,7 +759,6 @@ class FITS_rec(np.recarray):
         This results in a reference cycle that cannot be broken since
         ndarrays do not participate in cyclic garbage collection.
         """
-
         base = field
         while True:
             self_base = self
@@ -786,7 +786,6 @@ class FITS_rec(np.recarray):
         Dispatches column attribute change notifications to individual methods
         for each attribute ``_update_column_<attr>``
         """
-
         method_name = f"_update_column_{attr}"
         if hasattr(self, method_name):
             # Right now this is so we can be lazy and not implement updaters
@@ -795,7 +794,6 @@ class FITS_rec(np.recarray):
 
     def _update_column_name(self, column, idx, old_name, name):
         """Update the dtype field names when a column name is changed."""
-
         dtype = self.dtype
         # Updating the names on the dtype should suffice
         dtype.names = dtype.names[:idx] + (name,) + dtype.names[idx + 1 :]
@@ -804,7 +802,6 @@ class FITS_rec(np.recarray):
         """Convert a raw table column to a bit array as specified by the
         FITS X format.
         """
-
         dummy = np.zeros(self.shape + (recformat.repeat,), dtype=np.bool_)
         _unwrapx(field, dummy, recformat.repeat)
         return dummy
@@ -813,7 +810,6 @@ class FITS_rec(np.recarray):
         """Convert a raw table column of FITS P or Q format descriptors
         to a VLA column with the array data returned from the heap.
         """
-
         if column.dim:
             vla_shape = tuple(
                 reversed(tuple(map(int, column.dim.strip("()").split(","))))
@@ -823,15 +819,15 @@ class FITS_rec(np.recarray):
 
         if raw_data is None:
             raise OSError(
-                "Could not find heap data for the {!r} variable-length "
-                "array column.".format(column.name)
+                f"Could not find heap data for the {column.name!r} variable-length "
+                "array column."
             )
 
         for idx in range(len(self)):
-            offset = field[idx, 1] + self._heapoffset
-            count = field[idx, 0]
+            offset = int(field[idx, 1]) + self._heapoffset
+            count = int(field[idx, 0])
 
-            if recformat.dtype == "a":
+            if recformat.dtype == "S":
                 dt = np.dtype(recformat.dtype + str(1))
                 arr_len = count * dt.itemsize
                 da = raw_data[offset : offset + arr_len].view(dt)
@@ -869,7 +865,6 @@ class FITS_rec(np.recarray):
         Special handling for ASCII table columns to convert columns containing
         numeric types to actual numeric arrays from the string representation.
         """
-
         format = column.format
         recformat = getattr(format, "recformat", ASCII2NUMPY[format[0]])
         # if the string = TNULL, return ASCIITNULL
@@ -897,10 +892,10 @@ class FITS_rec(np.recarray):
         try:
             dummy = np.array(dummy, dtype=recformat)
         except ValueError as exc:
-            indx = self.names.index(column.name)
             raise ValueError(
-                "{}; the header may be missing the necessary TNULL{} "
-                "keyword or the table contains invalid data".format(exc, indx + 1)
+                f"{exc}; the header may be missing the necessary "
+                f"TNULL{self.names.index(column.name) + 1} keyword or the table "
+                "contains invalid data"
             )
 
         return dummy
@@ -911,23 +906,14 @@ class FITS_rec(np.recarray):
         This may not perform any conversion at all if it's not necessary, in
         which case the original column array is returned.
         """
-
         if isinstance(recformat, _FormatX):
             # special handling for the X format
             return self._convert_x(field, recformat)
 
-        (
-            _str,
-            _bool,
-            _number,
-            _scale,
-            _zero,
-            bscale,
-            bzero,
-            dim,
-        ) = self._get_scale_factors(column)
+        scale_factors = self._get_scale_factors(column)
+        _str, _bool, _number, _scale, _zero, bscale, bzero, dim = scale_factors
 
-        indx = self.names.index(column.name)
+        index = self.names.index(column.name)
 
         # ASCII table, convert strings to numbers
         # TODO:
@@ -959,18 +945,16 @@ class FITS_rec(np.recarray):
                 nitems = reduce(operator.mul, dim)
                 if _str:
                     actual_nitems = field.itemsize
-                elif (
-                    len(field.shape) == 1
-                ):  # No repeat count in TFORMn, equivalent to 1
+                elif len(field.shape) == 1:
+                    # No repeat count in TFORMn, equivalent to 1
                     actual_nitems = 1
                 else:
                     actual_nitems = field.shape[1]
                 if nitems > actual_nitems and not isinstance(recformat, _FormatP):
                     warnings.warn(
-                        "TDIM{} value {:d} does not fit with the size of "
-                        "the array items ({:d}).  TDIM{:d} will be ignored.".format(
-                            indx + 1, self._coldefs[indx].dims, actual_nitems, indx + 1
-                        )
+                        f"TDIM{index + 1} value {self._coldefs[index].dims:d} does not "
+                        f"fit with the size of the array items ({actual_nitems:d}).  "
+                        f"TDIM{index + 1:d} will be ignored."
                     )
                     dim = None
 
@@ -1016,8 +1000,8 @@ class FITS_rec(np.recarray):
                         test_overflow += bzero64
                     except OverflowError:
                         warnings.warn(
-                            "Overflow detected while applying TZERO{:d}. "
-                            "Returning unscaled data.".format(indx + 1)
+                            f"Overflow detected while applying TZERO{index + 1:d}. "
+                            "Returning unscaled data."
                         )
                     else:
                         field = test_overflow
@@ -1054,13 +1038,32 @@ class FITS_rec(np.recarray):
 
         This is returned as a numpy byte array.
         """
-
-        if self._heapsize:
-            raw_data = self._get_raw_data().view(np.ubyte)
+        raw_data = self._get_raw_data()
+        if self._heapsize and raw_data is not None:
+            # Read the heap from disk
+            raw_data = raw_data.view(np.ubyte)
             heap_end = self._heapoffset + self._heapsize
             return raw_data[self._heapoffset : heap_end]
         else:
-            return np.array([], dtype=np.ubyte)
+            # Data is only in memory so create the heap data, one column
+            # at a time, in the order that the data pointers appear in the
+            # column (regardless if that data pointer has a different,
+            # previous heap offset listed)
+            data = []
+            for idx in range(self._nfields):
+                # data should already be byteswapped from the caller
+                # using _binary_table_byte_swap
+                if not isinstance(self.columns._recformats[idx], _FormatP):
+                    continue
+
+                for row in self.field(idx):
+                    if len(row) > 0:
+                        data.append(row.view(type=np.ndarray, dtype=np.ubyte))
+
+            if data:
+                return np.concatenate(data)
+            else:
+                return np.array([], dtype=np.ubyte)
 
     def _get_raw_data(self):
         """
@@ -1078,8 +1081,7 @@ class FITS_rec(np.recarray):
         May return ``None`` if no array resembling the "raw data" according to
         the stated criteria can be found.
         """
-
-        raw_data_bytes = self.nbytes + self._heapsize
+        raw_data_bytes = self._tbsize + self._heapsize
         base = self
         while hasattr(base, "base") and base.base is not None:
             base = base.base
@@ -1095,7 +1097,6 @@ class FITS_rec(np.recarray):
 
     def _get_scale_factors(self, column):
         """Get all the scaling flags and factors for one column."""
-
         # TODO: Maybe this should be a method/property on Column?  Or maybe
         # it's not really needed at all...
         _str = column.format.format == "A"
@@ -1131,21 +1132,20 @@ class FITS_rec(np.recarray):
         the heap.  Currently this is only used as an optimization for
         CompImageHDU that does its own handling of the heap.
         """
-
         # Running total for the new heap size
         heapsize = 0
 
-        for indx, name in enumerate(self.dtype.names):
-            column = self._coldefs[indx]
+        for index, name in enumerate(self.dtype.names):
+            column = self._coldefs[index]
             recformat = column.format.recformat
-            raw_field = _get_recarray_field(self, indx)
+            raw_field = _get_recarray_field(self, index)
 
             # add the location offset of the heap area for each
             # variable length column
             if isinstance(recformat, _FormatP):
                 # Irritatingly, this can return a different dtype than just
                 # doing np.dtype(recformat.dtype); but this returns the results
-                # that we want.  For example if recformat.dtype is 'a' we want
+                # that we want.  For example if recformat.dtype is 'S' we want
                 # an array of characters.
                 dtype = np.array([], dtype=recformat.dtype).dtype
 
@@ -1165,27 +1165,13 @@ class FITS_rec(np.recarray):
                 # Even if this VLA has not been read or updated, we need to
                 # include the size of its constituent arrays in the heap size
                 # total
-                if heapsize >= 2**31:
-                    raise ValueError(
-                        "The heapsize limit for 'P' format "
-                        "has been reached. "
-                        "Please consider using the 'Q' format "
-                        "for your file."
-                    )
+
             if isinstance(recformat, _FormatX) and name in self._converted:
                 _wrapx(self._converted[name], raw_field, recformat.repeat)
                 continue
 
-            (
-                _str,
-                _bool,
-                _number,
-                _scale,
-                _zero,
-                bscale,
-                bzero,
-                _,
-            ) = self._get_scale_factors(column)
+            scale_factors = self._get_scale_factors(column)
+            _str, _bool, _number, _scale, _zero, bscale, bzero, _ = scale_factors
 
             field = self._converted.get(name, raw_field)
 
@@ -1194,7 +1180,8 @@ class FITS_rec(np.recarray):
                 if _number and (_scale or _zero) and column._physical_values:
                     dummy = field.copy()
                     if _zero:
-                        dummy -= bzero
+                        # Cast before subtracting to avoid overflow problems.
+                        dummy -= np.array(bzero).astype(dummy.dtype, casting="unsafe")
                     if _scale:
                         dummy /= bscale
                     # This will set the raw values in the recarray back to
@@ -1208,10 +1195,10 @@ class FITS_rec(np.recarray):
 
                 # ASCII table, convert numbers to strings
                 if isinstance(self._coldefs, _AsciiColDefs):
-                    self._scale_back_ascii(indx, dummy, raw_field)
+                    self._scale_back_ascii(index, dummy, raw_field)
                 # binary table string column
                 elif isinstance(raw_field, chararray.chararray):
-                    self._scale_back_strings(indx, dummy, raw_field)
+                    self._scale_back_strings(index, dummy, raw_field)
                 # all other binary table columns
                 else:
                     if len(raw_field) and isinstance(raw_field[0], np.integer):
@@ -1262,14 +1249,11 @@ class FITS_rec(np.recarray):
                 _ascii_encode(input_field, out=output_field)
             except _UnicodeArrayEncodeError as exc:
                 raise ValueError(
-                    "Could not save column '{}': Contains characters that "
-                    "cannot be encoded as ASCII as required by FITS, starting "
-                    "at the index {!r} of the column, and the index {} of "
-                    "the string at that location.".format(
-                        self._coldefs[col_idx].name,
-                        exc.index[0] if len(exc.index) == 1 else exc.index,
-                        exc.start,
-                    )
+                    f"Could not save column '{self._coldefs[col_idx].name}': "
+                    "Contains characters that cannot be encoded as ASCII as required "
+                    "by FITS, starting at the index "
+                    f"{exc.index[0] if len(exc.index) == 1 else exc.index!r} of the "
+                    f"column, and the index {exc.start} of the string at that location."
                 )
         else:
             # Otherwise go ahead and do a direct copy into--if both are type
@@ -1290,12 +1274,11 @@ class FITS_rec(np.recarray):
         the ``output_field`` is the character array representing the ASCII
         output that will be written.
         """
-
         starts = self._coldefs.starts[:]
         spans = self._coldefs.spans
         format = self._coldefs[col_idx].format
 
-        # The the index of the "end" column of the record, beyond
+        # The index of the "end" column of the record, beyond
         # which we can't write
         end = super().field(-1).itemsize
         starts.append(end + starts[-1])
@@ -1307,16 +1290,14 @@ class FITS_rec(np.recarray):
 
         if lead < 0:
             warnings.warn(
-                "Column {!r} starting point overlaps the previous column.".format(
-                    col_idx + 1
-                )
+                f"Column {col_idx + 1} starting point overlaps the previous column."
             )
 
         trail = starts[col_idx + 1] - starts[col_idx] - spans[col_idx]
 
         if trail < 0:
             warnings.warn(
-                f"Column {col_idx + 1!r} ending point overlaps the next column."
+                f"Column {col_idx + 1} ending point overlaps the next column."
             )
 
         # TODO: It would be nice if these string column formatting
@@ -1340,9 +1321,8 @@ class FITS_rec(np.recarray):
             value = fmt.format(value)
             if len(value) > starts[col_idx + 1] - starts[col_idx]:
                 raise ValueError(
-                    "Value {!r} does not fit into the output's itemsize of {}.".format(
-                        value, spans[col_idx]
-                    )
+                    f"Value {value!r} does not fit into the output's itemsize of "
+                    f"{spans[col_idx]}."
                 )
 
             if trailing_decimal and value[0] == " ":
@@ -1370,7 +1350,6 @@ def _get_recarray_field(array, key):
     This incorporates the legacy functionality of returning string arrays as
     Numeric-style chararray objects.
     """
-
     # Numpy >= 1.10.dev recarray no longer returns chararrays for strings
     # This is currently needed for backwards-compatibility and for
     # automatic truncation of trailing whitespace
@@ -1400,7 +1379,6 @@ def _ascii_encode(inarray, out=None):
     just a `UnicodeEncodeError` with an additional attribute for the index of
     the item that couldn't be encoded.
     """
-
     out_dtype = np.dtype((f"S{inarray.dtype.itemsize // 4}", inarray.dtype.shape))
     if out is not None:
         out = out.view(out_dtype)
@@ -1425,6 +1403,5 @@ def _has_unicode_fields(array):
     """
     Returns True if any fields in a structured array have Unicode dtype.
     """
-
     dtypes = (d[0] for d in array.dtype.fields.values())
     return any(d.kind == "U" for d in dtypes)
