@@ -1,6 +1,8 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
 # pylint: disable=invalid-name
+from contextlib import nullcontext
+
 import numpy as np
 import pytest
 from numpy.testing import assert_allclose, assert_array_equal, assert_array_less
@@ -8,14 +10,20 @@ from numpy.testing import assert_allclose, assert_array_equal, assert_array_less
 from astropy import units as u
 from astropy.coordinates import Angle
 from astropy.modeling import InputParameterError, fitting, models
-from astropy.utils.compat.optional_deps import HAS_SCIPY  # noqa: F401
-from astropy.utils.exceptions import AstropyUserWarning
+from astropy.utils.compat.optional_deps import HAS_SCIPY
+from astropy.utils.exceptions import AstropyDeprecationWarning, AstropyUserWarning
 
 fitters = [
     fitting.LevMarLSQFitter,
     fitting.TRFLSQFitter,
     fitting.LMLSQFitter,
-    fitting.DogBoxLSQFitter
+    fitting.DogBoxLSQFitter,
+]
+
+fitters_bounds = [
+    fitting.LevMarLSQFitter,
+    fitting.TRFLSQFitter,
+    fitting.DogBoxLSQFitter,
 ]
 
 
@@ -28,6 +36,7 @@ def test_sigma_constant():
     """
     from astropy.modeling.functional_models import GAUSSIAN_SIGMA_TO_FWHM
     from astropy.stats.funcs import gaussian_sigma_to_fwhm
+
     assert gaussian_sigma_to_fwhm == GAUSSIAN_SIGMA_TO_FWHM
 
 
@@ -37,12 +46,11 @@ def test_Trapezoid1D():
     model = models.Trapezoid1D(amplitude=4.2, x_0=2.0, width=1.0, slope=3)
     xx = np.linspace(0, 4, 8)
     yy = model(xx)
-    yy_ref = [0., 1.41428571, 3.12857143, 4.2, 4.2, 3.12857143, 1.41428571, 0.]
+    yy_ref = [0.0, 1.41428571, 3.12857143, 4.2, 4.2, 3.12857143, 1.41428571, 0.0]
     assert_allclose(yy, yy_ref, rtol=0, atol=1e-6)
 
 
 def test_Gaussian1D():
-
     model = models.Gaussian1D(4.2, 1.7, stddev=5.1)
     x = np.mgrid[0:5]
     g = model(x)
@@ -57,18 +65,22 @@ def test_Gaussian2D():
     https://github.com/astropy/astropy/pull/2038
     """
 
-    model = models.Gaussian2D(4.2, 1.7, 3.1, x_stddev=5.1, y_stddev=3.3,
-                              theta=np.pi/6.)
+    model = models.Gaussian2D(
+        4.2, 1.7, 3.1, x_stddev=5.1, y_stddev=3.3, theta=np.pi / 6.0
+    )
     y, x = np.mgrid[0:5, 0:5]
     g = model(x, y)
-    g_ref = [[3.01907812, 2.99051889, 2.81271552, 2.5119566, 2.13012709],
-             [3.55982239, 3.6086023, 3.4734158, 3.17454575, 2.75494838],
-             [3.88059142, 4.0257528, 3.96554926, 3.70908389, 3.29410187],
-             [3.91095768, 4.15212857, 4.18567526, 4.00652015, 3.64146544],
-             [3.6440466, 3.95922417, 4.08454159, 4.00113878, 3.72161094]]
+    g_ref = [
+        [3.01907812, 2.99051889, 2.81271552, 2.5119566, 2.13012709],
+        [3.55982239, 3.6086023, 3.4734158, 3.17454575, 2.75494838],
+        [3.88059142, 4.0257528, 3.96554926, 3.70908389, 3.29410187],
+        [3.91095768, 4.15212857, 4.18567526, 4.00652015, 3.64146544],
+        [3.6440466, 3.95922417, 4.08454159, 4.00113878, 3.72161094],
+    ]
     assert_allclose(g, g_ref, rtol=0, atol=1e-6)
-    assert_allclose([model.x_fwhm, model.y_fwhm],
-                    [12.009582229657841, 7.7709061486021325])
+    assert_allclose(
+        [model.x_fwhm, model.y_fwhm], [12.009582229657841, 7.7709061486021325]
+    )
 
 
 def test_Gaussian2DCovariance():
@@ -77,33 +89,38 @@ def test_Gaussian2DCovariance():
     https://github.com/astropy/astropy/pull/2199
     """
 
-    cov_matrix = [[49., -16.], [-16., 9.]]
-    model = models.Gaussian2D(17., 2.0, 2.5, cov_matrix=cov_matrix)
+    cov_matrix = [[49.0, -16.0], [-16.0, 9.0]]
+    model = models.Gaussian2D(17.0, 2.0, 2.5, cov_matrix=cov_matrix)
     y, x = np.mgrid[0:5, 0:5]
     g = model(x, y)
-    g_ref = [[4.3744505, 5.8413977, 7.42988694, 9.00160175, 10.38794269],
-             [8.83290201, 10.81772851, 12.61946384, 14.02225593, 14.84113227],
-             [13.68528889, 15.37184621, 16.44637743, 16.76048705, 16.26953638],
-             [16.26953638, 16.76048705, 16.44637743, 15.37184621, 13.68528889],
-             [14.84113227, 14.02225593, 12.61946384, 10.81772851, 8.83290201]]
+    g_ref = [
+        [4.3744505, 5.8413977, 7.42988694, 9.00160175, 10.38794269],
+        [8.83290201, 10.81772851, 12.61946384, 14.02225593, 14.84113227],
+        [13.68528889, 15.37184621, 16.44637743, 16.76048705, 16.26953638],
+        [16.26953638, 16.76048705, 16.44637743, 15.37184621, 13.68528889],
+        [14.84113227, 14.02225593, 12.61946384, 10.81772851, 8.83290201],
+    ]
     assert_allclose(g, g_ref, rtol=0, atol=1e-6)
 
     # Test bad cov_matrix shape
-    cov_matrix = [[49., 3.14, -16.],
-                  [3.14, -16., 9.],
-                  [-16, 27, 3.14]]
-    with pytest.raises(ValueError) as err:
-        models.Gaussian2D(17., 2.0, 2.5, cov_matrix=cov_matrix)
-    assert str(err.value) == "Covariance matrix must be 2x2"
+    cov_matrix = [[49.0, 3.14, -16.0], [3.14, -16.0, 9.0], [-16, 27, 3.14]]
+    MESSAGE = r"Covariance matrix must be 2x2"
+    with pytest.raises(ValueError, match=MESSAGE):
+        models.Gaussian2D(17.0, 2.0, 2.5, cov_matrix=cov_matrix)
 
 
 def test_Gaussian2DRotation():
     amplitude = 42
     x_mean, y_mean = 0, 0
     x_stddev, y_stddev = 2, 3
-    theta = Angle(10, 'deg')
-    pars = dict(amplitude=amplitude, x_mean=x_mean, y_mean=y_mean,
-                x_stddev=x_stddev, y_stddev=y_stddev)
+    theta = Angle(10, "deg")
+    pars = dict(
+        amplitude=amplitude,
+        x_mean=x_mean,
+        y_mean=y_mean,
+        x_stddev=x_stddev,
+        y_stddev=y_stddev,
+    )
     rotation = models.Rotation2D(angle=theta.degree)
     point1 = (x_mean + 2 * x_stddev, y_mean + 2 * y_stddev)
     point2 = rotation(*point1)
@@ -118,7 +135,7 @@ def test_Gaussian2D_invalid_inputs():
     x_stddev = 5.1
     y_stddev = 3.3
     theta = 10
-    cov_matrix = [[49., -16.], [-16., 9.]]
+    cov_matrix = [[49.0, -16.0], [-16.0, 9.0]]
 
     # first make sure the valid ones are OK
     models.Gaussian2D()
@@ -128,31 +145,32 @@ def test_Gaussian2D_invalid_inputs():
     models.Gaussian2D(x_stddev=x_stddev, y_stddev=y_stddev, theta=None)
     models.Gaussian2D(cov_matrix=cov_matrix)
 
-    with pytest.raises(InputParameterError):
+    MESSAGE = r"Cannot specify both cov_matrix and x/y_stddev/theta"
+    with pytest.raises(InputParameterError, match=MESSAGE):
         models.Gaussian2D(x_stddev=0, cov_matrix=cov_matrix)
-    with pytest.raises(InputParameterError):
+    with pytest.raises(InputParameterError, match=MESSAGE):
         models.Gaussian2D(y_stddev=0, cov_matrix=cov_matrix)
-    with pytest.raises(InputParameterError):
+    with pytest.raises(InputParameterError, match=MESSAGE):
         models.Gaussian2D(theta=0, cov_matrix=cov_matrix)
 
 
 def test_Gaussian2D_theta():
-    theta = Angle(90, 'deg')
+    theta = Angle(90, "deg")
     model1 = models.Gaussian2D(1, 25, 25, 15, 5, theta=theta)
 
-    theta2 = np.pi / 2.
+    theta2 = np.pi / 2.0
     model2 = models.Gaussian2D(1, 25, 25, 15, 5, theta=theta2)
 
-    assert model1.theta.quantity.to('radian').value == model2.theta.value
+    assert model1.theta.quantity.to("radian").value == model2.theta.value
     assert model1.bounding_box == model2.bounding_box
 
     assert model1(619.42, 31.314) == model2(619.42, 31.314)
 
 
-@pytest.mark.parametrize('gamma', (10, -10))
+@pytest.mark.parametrize("gamma", (10, -10))
 def test_moffat_fwhm(gamma):
     ans = 34.641016151377542
-    kwargs = {'gamma': gamma, 'alpha': 0.5}
+    kwargs = {"gamma": gamma, "alpha": 0.5}
     m1 = models.Moffat1D(**kwargs)
     m2 = models.Moffat2D(**kwargs)
     assert_allclose([m1.fwhm, m2.fwhm], ans)
@@ -172,11 +190,11 @@ def test_RedshiftScaleFactor():
     # Scale by a list
     m = models.RedshiftScaleFactor([-0.5, 0, 0.5], n_models=3)
     assert_array_equal(m(0), 0)
-    assert_array_equal(m([1, 2], model_set_axis=False),
-                       [[0.5, 1], [1, 2], [1.5, 3]])
+    assert_array_equal(m([1, 2], model_set_axis=False), [[0.5, 1], [1, 2], [1.5, 3]])
 
-    assert_allclose(m.inverse(m([1, 2], model_set_axis=False)),
-                    [[1, 2], [1, 2], [1, 2]])
+    assert_allclose(
+        m.inverse(m([1, 2], model_set_axis=False)), [[1, 2], [1, 2], [1, 2]]
+    )
 
 
 def test_RedshiftScaleFactor_inverse():
@@ -191,10 +209,12 @@ def test_RedshiftScaleFactor_inverse_bounding_box():
 
     inverse_model = model.inverse
     assert inverse_model.bounding_box == (3, 15)
-    assert_allclose(inverse_model(model(4, with_bounding_box=True), with_bounding_box=True), 4)
+    assert_allclose(
+        inverse_model(model(4, with_bounding_box=True), with_bounding_box=True), 4
+    )
 
 
-@pytest.mark.skipif('not HAS_SCIPY')
+@pytest.mark.skipif(not HAS_SCIPY, reason="requires scipy")
 def test_RedshiftScaleFactor_model_levmar_fit():
     """Test fitting RedshiftScaleFactor model with LevMarLSQFitter."""
 
@@ -213,7 +233,7 @@ def test_Ellipse2D():
     """Test Ellipse2D model."""
     amplitude = 7.5
     x0, y0 = 15, 15
-    theta = Angle(45, 'deg')
+    theta = Angle(45, "deg")
     em = models.Ellipse2D(amplitude, x0, y0, 7, 3, theta.radian)
     y, x = np.mgrid[0:30, 0:30]
     e = em(x, y)
@@ -221,11 +241,11 @@ def test_Ellipse2D():
     assert e[y0, x0] == amplitude
 
     rotation = models.Rotation2D(angle=theta.degree)
-    point1 = [2, 0]      # Rotation2D center is (0, 0)
+    point1 = [2, 0]  # Rotation2D center is (0, 0)
     point2 = rotation(*point1)
     point1 = np.array(point1) + [x0, y0]
     point2 = np.array(point2) + [x0, y0]
-    e1 = models.Ellipse2D(amplitude, x0, y0, 7, 3, theta=0.)
+    e1 = models.Ellipse2D(amplitude, x0, y0, 7, 3, theta=0.0)
     e2 = models.Ellipse2D(amplitude, x0, y0, 7, 3, theta=theta.radian)
     assert e1(*point1) == e2(*point2)
 
@@ -236,20 +256,19 @@ def test_Ellipse2D_circular():
     radius = 10
     size = (radius * 2) + 1
     y, x = np.mgrid[0:size, 0:size]
-    ellipse = models.Ellipse2D(amplitude, radius, radius, radius, radius,
-                               theta=0)(x, y)
+    ellipse = models.Ellipse2D(amplitude, radius, radius, radius, radius, theta=0)(x, y)
     disk = models.Disk2D(amplitude, radius, radius, radius)(x, y)
     assert np.all(ellipse == disk)
 
 
 def test_Ellipse2D_theta():
-    theta = Angle(90, 'deg')
+    theta = Angle(90, "deg")
     model1 = models.Ellipse2D(1, 25, 25, 15, 5, theta=theta)
 
-    theta2 = np.pi / 2.
+    theta2 = np.pi / 2.0
     model2 = models.Ellipse2D(1, 25, 25, 15, 5, theta=theta2)
 
-    assert model1.theta.quantity.to('radian').value == model2.theta.value
+    assert model1.theta.quantity.to("radian").value == model2.theta.value
     assert model1.bounding_box == model2.bounding_box
 
     assert model1(619.42, 31.314) == model2(619.42, 31.314)
@@ -267,7 +286,9 @@ def test_Scale_inverse_bounding_box():
 
     inverse_model = model.inverse
     assert inverse_model.bounding_box == (2, 10)
-    assert inverse_model(model(4, with_bounding_box=True), with_bounding_box=True) == 4.0
+    assert (
+        inverse_model(model(4, with_bounding_box=True), with_bounding_box=True) == 4.0
+    )
 
 
 def test_Multiply_inverse():
@@ -282,7 +303,9 @@ def test_Multiply_inverse_bounding_box():
 
     inverse_model = model.inverse
     assert inverse_model.bounding_box == (2, 10)
-    assert inverse_model(model(4, with_bounding_box=True), with_bounding_box=True) == 4.0
+    assert (
+        inverse_model(model(4, with_bounding_box=True), with_bounding_box=True) == 4.0
+    )
 
 
 def test_Shift_inverse():
@@ -297,11 +320,13 @@ def test_Shift_inverse_bounding_box():
 
     inverse_model = model.inverse
     assert inverse_model.bounding_box == (11, 15)
-    assert inverse_model(model(4, with_bounding_box=True), with_bounding_box=True) == 4.0
+    assert (
+        inverse_model(model(4, with_bounding_box=True), with_bounding_box=True) == 4.0
+    )
 
 
-@pytest.mark.skipif('not HAS_SCIPY')
-@pytest.mark.parametrize('fitter', fitters)
+@pytest.mark.skipif(not HAS_SCIPY, reason="requires scipy")
+@pytest.mark.parametrize("fitter", fitters)
 def test_Shift_model_levmar_fit(fitter):
     """Test fitting Shift model with LevMarLSQFitter (issue #6103)."""
     fitter = fitter()
@@ -311,8 +336,7 @@ def test_Shift_model_levmar_fit(fitter):
     x = np.arange(10)
     y = x + 0.1
 
-    with pytest.warns(AstropyUserWarning,
-                      match='Model is linear in parameters'):
+    with pytest.warns(AstropyUserWarning, match="Model is linear in parameters"):
         fitted_model = fitter(init_model, x, y)
 
     assert_allclose(fitted_model.parameters, [0.1], atol=1e-15)
@@ -324,7 +348,7 @@ def test_Shift_model_set_linear_fit():
     init_model = models.Shift(offset=[0, 0], n_models=2)
 
     x = np.arange(10)
-    yy = np.array([x+0.1, x-0.2])
+    yy = np.array([x + 0.1, x - 0.2])
 
     fitter = fitting.LinearLSQFitter()
     fitted_model = fitter(init_model, x, yy)
@@ -332,14 +356,14 @@ def test_Shift_model_set_linear_fit():
     assert_allclose(fitted_model.parameters, [0.1, -0.2], atol=1e-15)
 
 
-@pytest.mark.parametrize('Model', (models.Scale, models.Multiply))
+@pytest.mark.parametrize("Model", (models.Scale, models.Multiply))
 def test_Scale_model_set_linear_fit(Model):
     """Test linear fitting of Scale model (#6103)."""
 
     init_model = Model(factor=[0, 0], n_models=2)
 
     x = np.arange(-3, 7)
-    yy = np.array([1.15*x, 0.96*x])
+    yy = np.array([1.15 * x, 0.96 * x])
 
     fitter = fitting.LinearLSQFitter()
     fitted_model = fitter(init_model, x, yy)
@@ -347,14 +371,14 @@ def test_Scale_model_set_linear_fit(Model):
     assert_allclose(fitted_model.parameters, [1.15, 0.96], atol=1e-15)
 
 
-@pytest.mark.parametrize('Model', (models.Scale, models.Multiply))
+@pytest.mark.parametrize("Model", (models.Scale, models.Multiply))
 def test_Scale_model_evaluate_without_units(Model):
-    m = Model(factor=4*u.m)
-    kwargs = {'x': 3*u.m, 'y': 7*u.m}
+    m = Model(factor=4 * u.m)
+    kwargs = {"x": 3 * u.m, "y": 7 * u.m}
     mnu = m.without_units_for_data(**kwargs)
 
     x = np.linspace(-1, 1, 100)
-    assert_allclose(mnu(x), 4*x)
+    assert_allclose(mnu(x), 4 * x)
 
 
 # https://github.com/astropy/astropy/issues/6178
@@ -383,9 +407,9 @@ def test_Ring2D_rout():
     assert m.r_in.value == 1
     assert m.width.value == 6
     # Error when r_out is too small for default r_in
-    with pytest.raises(InputParameterError) as err:
+    MESSAGE = r"r_in=.* and width=.* must both be >=0"
+    with pytest.raises(InputParameterError, match=MESSAGE):
         models.Ring2D(amplitude=1, x_0=1, y_0=1, r_out=0.5)
-    assert str(err.value) == "r_in=1 and width=-0.5 must both be >=0"
 
     # Test with width specified only
     m = models.Ring2D(amplitude=1, x_0=1, y_0=1, width=11)
@@ -403,9 +427,8 @@ def test_Ring2D_rout():
     assert m.r_in.value == 2
     assert m.width.value == 3
     # Error when r_out is smaller than r_in
-    with pytest.raises(InputParameterError) as err:
+    with pytest.raises(InputParameterError, match=MESSAGE):
         models.Ring2D(amplitude=1, x_0=1, y_0=1, r_out=1, r_in=4)
-    assert str(err.value) == "r_in=4 and width=-3 must both be >=0"
 
     # Test with r_in and width specified only
     m = models.Ring2D(amplitude=1, x_0=1, y_0=1, r_in=2, width=4)
@@ -423,9 +446,8 @@ def test_Ring2D_rout():
     assert m.r_in.value == 5
     assert m.width.value == 7
     # Error when width is larger than r_out
-    with pytest.raises(InputParameterError) as err:
+    with pytest.raises(InputParameterError, match=MESSAGE):
         models.Ring2D(amplitude=1, x_0=1, y_0=1, r_out=1, width=4)
-    assert str(err.value) == "r_in=-3 and width=4 must both be >=0"
 
     # Test with r_in, r_out, and width all specified
     m = models.Ring2D(amplitude=1, x_0=1, y_0=1, r_in=3, r_out=11, width=8)
@@ -435,13 +457,13 @@ def test_Ring2D_rout():
     assert m.r_in.value == 3
     assert m.width.value == 8
     # error when specifying all
-    with pytest.raises(InputParameterError) as err:
+    MESSAGE = r"Width must be r_out - r_in"
+    with pytest.raises(InputParameterError, match=MESSAGE):
         models.Ring2D(amplitude=1, x_0=1, y_0=1, r_in=3, r_out=11, width=7)
-    assert str(err.value) == "Width must be r_out - r_in"
 
 
-@pytest.mark.skipif("not HAS_SCIPY")
-@pytest.mark.parametrize('fitter', fitters)
+@pytest.mark.skipif(not HAS_SCIPY, reason="requires scipy")
+@pytest.mark.parametrize("fitter", fitters)
 def test_Voigt1D(fitter):
     fitter = fitter()
 
@@ -453,45 +475,90 @@ def test_Voigt1D(fitter):
     assert_allclose(voi_fit.param_sets, voi.param_sets)
 
     # Invalid method
-    with pytest.raises(ValueError) as err:
-        models.Voigt1D(method='test')
-    assert str(err.value) == "Not a valid method for Voigt1D Faddeeva function: test."
+    MESSAGE = r"Not a valid method for Voigt1D Faddeeva function: test"
+    with pytest.raises(ValueError, match=MESSAGE):
+        models.Voigt1D(method="test")
 
 
-@pytest.mark.skipif("not HAS_SCIPY")
-@pytest.mark.parametrize('algorithm', ('humlicek2', 'wofz'))
+@pytest.mark.skipif(not HAS_SCIPY, reason="requires scipy")
+@pytest.mark.parametrize("algorithm", ("humlicek2", "wofz"))
 def test_Voigt1D_norm(algorithm):
     """Test integral of normalized Voigt profile."""
     from scipy.integrate import quad
 
-    voi = models.Voigt1D(amplitude_L=1.0/np.pi, x_0=0.0, fwhm_L=2.0, fwhm_G=1.5, method=algorithm)
-    if algorithm == 'wofz':
-        atol = 1e-14
-    else:
+    if algorithm == "humlicek2":
+        ctx = pytest.warns(
+            AstropyDeprecationWarning, match=r"humlicek2 has been deprecated since .*"
+        )
         atol = 1e-8
+    else:
+        ctx = nullcontext()
+        atol = 1e-14
+
+    def voigt(algorithm):
+        return models.Voigt1D(
+            amplitude_L=1.0 / np.pi, x_0=0.0, fwhm_L=2.0, fwhm_G=1.5, method=algorithm
+        )
+
+    with ctx:
+        voi = models.Voigt1D(
+            amplitude_L=1.0 / np.pi, x_0=0.0, fwhm_L=2.0, fwhm_G=1.5, method=algorithm
+        )
+
     assert_allclose(quad(voi, -np.inf, np.inf)[0], 1.0, atol=atol)
 
 
-@pytest.mark.skipif("not HAS_SCIPY")
-@pytest.mark.parametrize('doppler', (1.e-3, 1.e-2, 0.1, 0.5, 1.0, 2.5, 5.0, 10))
+@pytest.mark.skipif(not HAS_SCIPY, reason="requires scipy")
+@pytest.mark.parametrize("doppler", (1.0e-3, 1.0e-2, 0.1, 0.5, 1.0, 2.5, 5.0, 10))
+@pytest.mark.filterwarnings(r"ignore:humlicek2 has been deprecated since .*")
 def test_Voigt1D_hum2(doppler):
-    """Verify accuracy of Voigt profile in Humlicek approximation to Faddeeva.cc (SciPy)."""
+    """
+    Verify accuracy of Voigt profile in Humlicek approximation to Faddeeva.cc (SciPy).
+    """
     x = np.linspace(-20, 20, 400001)
 
-    voi_w = models.Voigt1D(amplitude_L=2.0/np.pi, fwhm_L=1.0, fwhm_G=doppler, method='wofz')
+    voi_w = models.Voigt1D(
+        amplitude_L=2.0 / np.pi, fwhm_L=1.0, fwhm_G=doppler, method="wofz"
+    )
     vf_w = voi_w(x)
-    dvda_w = voi_w.fit_deriv(x, x_0=0, amplitude_L=2.0/np.pi, fwhm_L=1.0, fwhm_G=doppler)
+    dvda_w = voi_w.fit_deriv(
+        x, x_0=0, amplitude_L=2.0 / np.pi, fwhm_L=1.0, fwhm_G=doppler
+    )
 
-    voi_h = models.Voigt1D(amplitude_L=2.0/np.pi, fwhm_L=1.0, fwhm_G=doppler, method='humlicek2')
+    voi_h = models.Voigt1D(
+        amplitude_L=2.0 / np.pi, fwhm_L=1.0, fwhm_G=doppler, method="humlicek2"
+    )
     vf_h = voi_h(x)
-    dvda_h = voi_h.fit_deriv(x, x_0=0, amplitude_L=2.0/np.pi, fwhm_L=1.0, fwhm_G=doppler)
+    dvda_h = voi_h.fit_deriv(
+        x, x_0=0, amplitude_L=2.0 / np.pi, fwhm_L=1.0, fwhm_G=doppler
+    )
 
     assert_allclose(vf_h, vf_w, rtol=1e-7 * (2 + 1 / np.sqrt(doppler)))
     assert_allclose(dvda_h, dvda_w, rtol=1e-9, atol=1e-7 * (1 + 30 / doppler))
 
 
-@pytest.mark.skipif("not HAS_SCIPY")
-@pytest.mark.parametrize('fitter', fitters)
+@pytest.mark.filterwarnings(r"ignore:humlicek2 has been deprecated since .*")
+def test_Voigt1D_method():
+    """Test Voigt1D default method"""
+
+    voi = models.Voigt1D(method="humlicek2")
+    assert voi.method == "_hum2zpf16c"
+
+    voi = models.Voigt1D()
+    if HAS_SCIPY:
+        assert voi.method == "wofz"
+
+        voi = models.Voigt1D(method="wofz")
+        assert voi.method == "wofz"
+
+        voi = models.Voigt1D(method="scipy")
+        assert voi.method == "wofz"
+    else:
+        assert voi.method == "_hum2zpf16c"
+
+
+@pytest.mark.skipif(not HAS_SCIPY, reason="requires scipy")
+@pytest.mark.parametrize("fitter", fitters_bounds)
 def test_KingProjectedAnalytic1D_fit(fitter):
     fitter = fitter()
 
@@ -504,35 +571,39 @@ def test_KingProjectedAnalytic1D_fit(fitter):
     assert_allclose(km_fit.concentration, 0.30102999566398136)
 
 
-@pytest.mark.parametrize('model', [models.Exponential1D(), models.Logarithmic1D()])
+@pytest.mark.parametrize("model", [models.Exponential1D(), models.Logarithmic1D()])
 def test_ExponentialAndLogarithmic1D_fit(model):
-    xarr = np.linspace(0.1, 10., 200)
+    xarr = np.linspace(0.1, 10.0, 200)
     assert_allclose(xarr, model.inverse(model(xarr)))
 
 
-@pytest.mark.parametrize('model', [models.Exponential1D(), models.Logarithmic1D()])
+@pytest.mark.parametrize("model", [models.Exponential1D(), models.Logarithmic1D()])
 def test_ExponentialAndLogarithmic_set_tau(model):
-    message = "0 is not an allowed value for tau"
+    MESSAGE = r"0 is not an allowed value for tau"
 
-    with pytest.raises(ValueError) as err:
+    with pytest.raises(ValueError, match=MESSAGE):
         model.tau = 0
-    assert str(err.value) == message
 
 
 def test_Linear1D_inverse():
     model = models.Linear1D(slope=4, intercept=-12)
     inverse = model.inverse
 
-    assert inverse.slope == 1/4
+    assert inverse.slope == 1 / 4
     assert inverse.intercept == 3
 
 
-@pytest.mark.parametrize('trig', [(models.Sine1D, [-0.25, 0.25]),
-                                  (models.ArcSine1D, [-0.25, 0.25]),
-                                  (models.Cosine1D, [0, 0.5]),
-                                  (models.ArcCosine1D, [0, 0.5]),
-                                  (models.Tangent1D, [-0.25, 0.25]),
-                                  (models.ArcTangent1D, [-0.25, 0.25])])
+@pytest.mark.parametrize(
+    "trig",
+    [
+        (models.Sine1D, [-0.25, 0.25]),
+        (models.ArcSine1D, [-0.25, 0.25]),
+        (models.Cosine1D, [0, 0.5]),
+        (models.ArcCosine1D, [0, 0.5]),
+        (models.Tangent1D, [-0.25, 0.25]),
+        (models.ArcTangent1D, [-0.25, 0.25]),
+    ],
+)
 def test_trig_inverse(trig):
     mdl = trig[0]()
     lower, upper = trig[1]
@@ -542,14 +613,14 @@ def test_trig_inverse(trig):
     assert_allclose(mdl(mdl.inverse(x)), x, atol=1e-10)
 
 
-@pytest.mark.skipif('not HAS_SCIPY')
+@pytest.mark.skipif(not HAS_SCIPY, reason="requires scipy")
 def test_Sersic2D_theta():
-    theta = Angle(90, 'deg')
+    theta = Angle(90, "deg")
     model1 = models.Sersic2D(1, 5, 4, 25, 25, 0.5, theta=theta)
 
-    theta2 = np.pi / 2.
+    theta2 = np.pi / 2.0
     model2 = models.Sersic2D(1, 5, 4, 25, 25, 0.5, theta=theta2)
 
-    assert model1.theta.quantity.to('radian').value == model2.theta.value
+    assert model1.theta.quantity.to("radian").value == model2.theta.value
 
     assert model1(619.42, 31.314) == model2(619.42, 31.314)
