@@ -1,18 +1,29 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
-"""Defines the physical types that correspond to different units."""
+"""Defines the physical types that correspond to different units.
+
+The classes and functions defined here are also available in
+(and should be used through) the `astropy.units` namespace.
+"""
+
+from __future__ import annotations
 
 import numbers
-import warnings
+from typing import TYPE_CHECKING
 
-from astropy.utils.exceptions import AstropyDeprecationWarning
+from astropy.utils.compat import COPY_IF_NEEDED
 
-from . import imperial  # for bkwd compat #11975 and #11977  # noqa: F401
 from . import astrophys, cgs, core, misc, quantity, si
 
-__all__ = ["def_physical_type", "get_physical_type", "PhysicalType"]
+if TYPE_CHECKING:
+    from collections.abc import Iterator
+    from typing import Final
 
-_units_and_physical_types = [
+    from .typing import PhysicalTypeID, QuantityLike, UnitPowerLike
+
+__all__: Final = ["PhysicalType", "def_physical_type", "get_physical_type"]
+
+_units_and_physical_types: Final[list[tuple[core.UnitBase, str | set[str]]]] = [
     (core.dimensionless_unscaled, "dimensionless"),
     (si.m, "length"),
     (si.m**2, "area"),
@@ -65,6 +76,7 @@ _units_and_physical_types = [
     (si.C / si.m**3, "electrical charge density"),
     (si.F / si.m, "permittivity"),
     (si.Wb, "magnetic flux"),
+    (si.Wb**2, "magnetic helicity"),
     (si.T, "magnetic flux density"),
     (si.A / si.m, "magnetic field strength"),
     (si.m**2 * si.A, "magnetic moment"),
@@ -77,10 +89,17 @@ _units_and_physical_types = [
     (si.cd / si.m**2, "luminance"),
     (si.m**-3 * si.s**-1, "volumetric rate"),
     (astrophys.Jy, "spectral flux density"),
+    (astrophys.Jy / si.sr, "surface brightness"),
     (si.W * si.m**2 * si.Hz**-1, "surface tension"),
     (si.J * si.m**-3 * si.s**-1, {"spectral flux density wav", "power density"}),
+    (si.J * si.m**-3 * si.s**-1 * si.sr**-1, "surface brightness wav"),
     (astrophys.photon / si.Hz / si.cm**2 / si.s, "photon flux density"),
     (astrophys.photon / si.AA / si.cm**2 / si.s, "photon flux density wav"),
+    (astrophys.photon / si.Hz / si.cm**2 / si.s / si.sr, "photon surface brightness"),
+    (
+        astrophys.photon / si.AA / si.cm**2 / si.s / si.sr,
+        "photon surface brightness wav",
+    ),
     (astrophys.R, "photon flux"),
     (misc.bit, "data quantity"),
     (misc.bit / si.s, "bandwidth"),
@@ -93,7 +112,7 @@ _units_and_physical_types = [
     (si.m * si.s**-5, "crackle"),
     (si.m * si.s**-6, {"pop", "pounce"}),
     (si.K / si.m, "temperature gradient"),
-    (si.J / si.kg, "specific energy"),
+    (si.J / si.kg, {"specific energy", "dose of ionizing radiation"}),
     (si.mol * si.m**-3 * si.s**-1, "reaction rate"),
     (si.kg * si.m**2, "moment of inertia"),
     (si.mol / si.s, "catalytic activity"),
@@ -124,14 +143,14 @@ _units_and_physical_types = [
     (si.m**-2 * si.s**-1, "particle flux"),
 ]
 
-_physical_unit_mapping = {}
-_unit_physical_mapping = {}
-_name_physical_mapping = {}
+_physical_unit_mapping: Final[dict[PhysicalTypeID, PhysicalType]] = {}
+_unit_physical_mapping: Final[dict[str, PhysicalTypeID]] = {}
+_name_physical_mapping: Final[dict[str, PhysicalType]] = {}
 # mapping from attribute-accessible name (no spaces, etc.) to the actual name.
-_attrname_physical_mapping = {}
+_attrname_physical_mapping: Final[dict[str, PhysicalType]] = {}
 
 
-def _physical_type_from_str(name):
+def _physical_type_from_str(name: str) -> PhysicalType:
     """
     Return the `PhysicalType` instance associated with the name of a
     physical type.
@@ -147,24 +166,23 @@ def _physical_type_from_str(name):
         raise ValueError(f"{name!r} is not a known physical type.")
 
 
-def _replace_temperatures_with_kelvin(unit):
-    """
-    If a unit contains a temperature unit besides kelvin, then replace
-    that unit with kelvin.
+def _replace_temperatures_with_kelvin(unit: core.UnitBase) -> core.UnitBase:
+    """Replace °F, and °C in the bases of `unit` with K.
 
-    Temperatures cannot be converted directly between K, °F, °C, and
-    °Ra, in particular since there would be different conversions for
-    T and ΔT.  However, each of these temperatures each represents the
-    physical type.  Replacing the different temperature units with
-    kelvin allows the physical type to be treated consistently.
+    The Kelvin, Celsius and Fahrenheit scales have different zero points,
+    which is a problem for the unit conversion machinery (without the
+    `temperature` equivalency). Replacing °F, and °C with kelvin allows the
+    physical type to be treated consistently. The Rankine scale has the
+    same zero point as the Kelvin scale, so degrees Rankine do not have to
+    be special-cased.
     """
-    physical_type_id = unit._get_physical_type_id()
+    physical_type_id = unit._physical_type_id
 
     physical_type_id_components = []
     substitution_was_made = False
 
     for base, power in physical_type_id:
-        if base in ["deg_F", "deg_C", "deg_R"]:
+        if base in ["deg_F", "deg_C"]:
             base = "K"
             substitution_was_made = True
         physical_type_id_components.append((base, power))
@@ -175,7 +193,7 @@ def _replace_temperatures_with_kelvin(unit):
         return unit
 
 
-def _standardize_physical_type_names(physical_type_input):
+def _standardize_physical_type_names(physical_type_input: str | set[str]) -> set[str]:
     """
     Convert a string or `set` of strings into a `set` containing
     string representations of physical types.
@@ -308,75 +326,40 @@ class PhysicalType:
     PhysicalType('length')
     """
 
-    def __init__(self, unit, physical_types):
+    def __init__(self, unit: core.UnitBase, physical_types: str | set[str]) -> None:
         self._unit = _replace_temperatures_with_kelvin(unit)
-        self._physical_type_id = self._unit._get_physical_type_id()
-        self._physical_type = _standardize_physical_type_names(physical_types)
-        self._physical_type_list = sorted(self._physical_type)
+        self._physical_type = sorted(_standardize_physical_type_names(physical_types))
 
-    def __iter__(self):
-        yield from self._physical_type_list
+    def __iter__(self) -> Iterator[str]:
+        yield from self._physical_type
 
-    def __getattr__(self, attr):
-        # TODO: remove this whole method when accessing str attributes from
-        # physical types is no longer supported
-
-        # short circuit attribute accessed in __str__ to prevent recursion
-        if attr == "_physical_type_list":
-            super().__getattribute__(attr)
-
-        self_str_attr = getattr(str(self), attr, None)
-        if hasattr(str(self), attr):
-            warning_message = (
-                f"support for accessing str attributes such as {attr!r} "
-                "from PhysicalType instances is deprecated since 4.3 "
-                "and will be removed in a subsequent release."
-            )
-            warnings.warn(warning_message, AstropyDeprecationWarning)
-            return self_str_attr
-        else:
-            super().__getattribute__(attr)  # to get standard error message
-
-    def __eq__(self, other):
+    def __eq__(self, other: object) -> bool:
         """
         Return `True` if ``other`` represents a physical type that is
         consistent with the physical type of the `PhysicalType` instance.
         """
         if isinstance(other, PhysicalType):
-            return self._physical_type_id == other._physical_type_id
+            return self._unit._physical_type_id == other._unit._physical_type_id
         elif isinstance(other, str):
             other = _standardize_physical_type_names(other)
             return other.issubset(self._physical_type)
         else:
             return NotImplemented
 
-    def __ne__(self, other):
-        equality = self.__eq__(other)
-        return not equality if isinstance(equality, bool) else NotImplemented
-
-    def _name_string_as_ordered_set(self):
-        return "{" + str(self._physical_type_list)[1:-1] + "}"
-
-    def __repr__(self):
+    def __repr__(self) -> str:
         if len(self._physical_type) == 1:
-            names = "'" + self._physical_type_list[0] + "'"
+            names = "'" + self._physical_type[0] + "'"
         else:
-            names = self._name_string_as_ordered_set()
+            names = "{" + str(self._physical_type)[1:-1] + "}"
         return f"PhysicalType({names})"
 
-    def __str__(self):
-        return "/".join(self._physical_type_list)
+    def __str__(self) -> str:
+        return "/".join(self._physical_type)
 
     @staticmethod
-    def _dimensionally_compatible_unit(obj):
+    def _dimensionally_compatible_unit(obj: object) -> core.UnitBase | None:
         """
         Return a unit that corresponds to the provided argument.
-
-        If a unit is passed in, return that unit.  If a physical type
-        (or a `str` with the name of a physical type) is passed in,
-        return a unit that corresponds to that physical type.  If the
-        number equal to ``1`` is passed in, return a dimensionless unit.
-        Otherwise, return `NotImplemented`.
         """
         if isinstance(obj, core.UnitBase):
             return _replace_temperatures_with_kelvin(obj)
@@ -386,39 +369,39 @@ class PhysicalType:
             return core.dimensionless_unscaled
         elif isinstance(obj, str):
             return _physical_type_from_str(obj)._unit
-        else:
-            return NotImplemented
+        return None
 
-    def _dimensional_analysis(self, other, operation):
-        other_unit = self._dimensionally_compatible_unit(other)
-        if other_unit is NotImplemented:
-            return NotImplemented
-        other_unit = _replace_temperatures_with_kelvin(other_unit)
-        new_unit = getattr(self._unit, operation)(other_unit)
-        return new_unit.physical_type
+    def __mul__(
+        self, other: PhysicalType | core.UnitBase | numbers.Real | str
+    ) -> PhysicalType:
+        if other_unit := self._dimensionally_compatible_unit(other):
+            return (self._unit * other_unit).physical_type
+        return NotImplemented
 
-    def __mul__(self, other):
-        return self._dimensional_analysis(other, "__mul__")
-
-    def __rmul__(self, other):
+    def __rmul__(self, other: PhysicalType | core.UnitBase | str) -> PhysicalType:
         return self.__mul__(other)
 
-    def __truediv__(self, other):
-        return self._dimensional_analysis(other, "__truediv__")
+    def __truediv__(
+        self, other: PhysicalType | core.UnitBase | numbers.Real | str
+    ) -> PhysicalType:
+        if other_unit := self._dimensionally_compatible_unit(other):
+            return (self._unit / other_unit).physical_type
+        return NotImplemented
 
-    def __rtruediv__(self, other):
-        other = self._dimensionally_compatible_unit(other)
-        if other is NotImplemented:
-            return NotImplemented
-        return other.physical_type._dimensional_analysis(self, "__truediv__")
+    def __rtruediv__(
+        self, other: PhysicalType | core.UnitBase | numbers.Real | str
+    ) -> PhysicalType:
+        if other_unit := self._dimensionally_compatible_unit(other):
+            return (other_unit / self._unit).physical_type
+        return NotImplemented
 
-    def __pow__(self, power):
+    def __pow__(self, power: UnitPowerLike) -> PhysicalType:
         return (self._unit**power).physical_type
 
-    def __hash__(self):
-        return hash(self._physical_type_id)
+    def __hash__(self) -> int:
+        return hash(self._unit._physical_type_id)
 
-    def __len__(self):
+    def __len__(self) -> int:
         return len(self._physical_type)
 
     # We need to prevent operations like where a Unit instance left
@@ -426,10 +409,10 @@ class PhysicalType:
     # instance with a PhysicalType as the value.  We can do this by
     # preventing np.array from casting a PhysicalType instance as
     # an object array.
-    __array__ = None
+    __array__: Final = None
 
 
-def def_physical_type(unit, name):
+def def_physical_type(unit: core.UnitBase, name: str | set[str]) -> None:
     """
     Add a mapping between a unit and the corresponding physical type(s).
 
@@ -457,7 +440,7 @@ def def_physical_type(unit, name):
     -----
     For a list of physical types, see `astropy.units.physical`.
     """
-    physical_type_id = unit._get_physical_type_id()
+    physical_type_id = unit._physical_type_id
     physical_type_names = _standardize_physical_type_names(name)
 
     if "unknown" in physical_type_names:
@@ -476,23 +459,22 @@ def def_physical_type(unit, name):
     unit_already_in_use = physical_type_id in _physical_unit_mapping
     if unit_already_in_use:
         physical_type = _physical_unit_mapping[physical_type_id]
-        physical_type_names |= set(physical_type)
-        physical_type.__init__(unit, physical_type_names)
+        physical_type._physical_type = sorted(physical_type_names | set(physical_type))
     else:
         physical_type = PhysicalType(unit, physical_type_names)
         _physical_unit_mapping[physical_type_id] = physical_type
 
     for ptype in physical_type:
         _unit_physical_mapping[ptype] = physical_type_id
-
-    for ptype_name in physical_type_names:
-        _name_physical_mapping[ptype_name] = physical_type
+        _name_physical_mapping[ptype] = physical_type
         # attribute-accessible name
-        attr_name = ptype_name.replace(" ", "_").replace("(", "").replace(")", "")
+        attr_name = ptype.replace(" ", "_").replace("(", "").replace(")", "")
         _attrname_physical_mapping[attr_name] = physical_type
 
 
-def get_physical_type(obj):
+def get_physical_type(
+    obj: PhysicalType | str | core.UnitBase | QuantityLike,
+) -> PhysicalType:
     """
     Return the physical type that corresponds to a unit (or another
     physical type representation).
@@ -549,12 +531,12 @@ def get_physical_type(obj):
         unit = obj
     else:
         try:
-            unit = quantity.Quantity(obj, copy=False).unit
+            unit = quantity.Quantity(obj, copy=COPY_IF_NEEDED).unit
         except TypeError as exc:
             raise TypeError(f"{obj} does not correspond to a physical type.") from exc
 
     unit = _replace_temperatures_with_kelvin(unit)
-    physical_type_id = unit._get_physical_type_id()
+    physical_type_id = unit._physical_type_id
     unit_has_known_physical_type = physical_type_id in _physical_unit_mapping
 
     if unit_has_known_physical_type:
@@ -569,6 +551,7 @@ def get_physical_type(obj):
 # define the physical types
 for unit, physical_type in _units_and_physical_types:
     def_physical_type(unit, physical_type)
+del unit, physical_type
 
 
 # For getting the physical types.
@@ -600,7 +583,7 @@ def __getattr__(name):
     raise AttributeError(f"module {__name__!r} has no attribute {name!r}")
 
 
-def __dir__():
+def __dir__() -> list[str]:
     """Return contents directory (__all__ + all physical type names)."""
     return list(set(__all__) | set(_attrname_physical_mapping.keys()))
 
@@ -619,14 +602,11 @@ if __doc__ is not None:
     ]
 
     for name in sorted(_name_physical_mapping.keys()):
-        physical_type = _name_physical_mapping[name]
+        ptype = _name_physical_mapping[name]
         doclines += [
             f"    * - _`{name}`",
-            f"      - :math:`{physical_type._unit.to_string('latex')[1:-1]}`",
-            f"      - {', '.join([n for n in physical_type if n != name])}",
+            f"      - :math:`{ptype._unit.to_string('latex')[1:-1]}`",
+            f"      - {', '.join([n for n in ptype if n != name])}",
         ]
 
     __doc__ += "\n\n" + "\n".join(doclines)
-
-
-del unit, physical_type
