@@ -1,27 +1,36 @@
-# -*- coding: utf-8 -*-
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """
 This module defines structured units and quantities.
 """
 
+from __future__ import annotations
+
 # Standard library
 import operator
+from functools import cached_property
+from typing import TYPE_CHECKING
 
 import numpy as np
 
+from astropy.utils.compat.numpycompat import NUMPY_LT_1_24
+
 from .core import UNITY, Unit, UnitBase
 
-__all__ = ['StructuredUnit']
+if TYPE_CHECKING:
+    from collections.abc import Collection
+    from typing import Self
+
+__all__ = ["StructuredUnit"]
 
 
-DTYPE_OBJECT = np.dtype('O')
+DTYPE_OBJECT = np.dtype("O")
 
 
 def _names_from_dtype(dtype):
     """Recursively extract field names from a dtype."""
     names = []
     for name in dtype.names:
-        subdtype = dtype.fields[name][0]
+        subdtype = dtype.fields[name][0].base
         if subdtype.names:
             names.append([name, _names_from_dtype(subdtype)])
         else:
@@ -41,19 +50,25 @@ def _normalize_names(names):
     for name in names:
         if isinstance(name, str) and len(name) > 0:
             result.append(name)
-        elif (isinstance(name, list)
-              and len(name) == 2
-              and isinstance(name[0], str) and len(name[0]) > 0
-              and isinstance(name[1], tuple) and len(name[1]) > 0):
+        elif (
+            isinstance(name, list)
+            and len(name) == 2
+            and isinstance(name[0], str)
+            and len(name[0]) > 0
+            and isinstance(name[1], tuple)
+            and len(name[1]) > 0
+        ):
             result.append([name[0], _normalize_names(name[1])])
         elif isinstance(name, tuple) and len(name) > 0:
             new_tuple = _normalize_names(name)
-            result.append([''.join([(i[0] if isinstance(i, list) else i)
-                                    for i in new_tuple]), new_tuple])
+            name = "".join([(i[0] if isinstance(i, list) else i) for i in new_tuple])
+            result.append([name, new_tuple])
         else:
-            raise ValueError(f'invalid entry {name!r}. Should be a name, '
-                             'tuple of names, or 2-element list of the '
-                             'form [name, tuple of names].')
+            raise ValueError(
+                f"invalid entry {name!r}. Should be a name, "
+                "tuple of names, or 2-element list of the "
+                "form [name, tuple of names]."
+            )
 
     return tuple(result)
 
@@ -77,7 +92,7 @@ class StructuredUnit:
 
     Notes
     -----
-    It is recommended to initialze the class indirectly, using
+    It is recommended to initialize the class indirectly, using
     `~astropy.units.Unit`.  E.g., ``u.Unit('AU,AU/day')``.
 
     When combined with a structured array to produce a structured
@@ -114,11 +129,12 @@ class StructuredUnit:
     Structured units share most methods with regular units::
 
         >>> su.physical_type
-        ((PhysicalType('length'), PhysicalType({'speed', 'velocity'})), PhysicalType('time'))
+        astropy.units.structured.Structure((astropy.units.structured.Structure((PhysicalType('length'), PhysicalType({'speed', 'velocity'})), dtype=[('f0', 'O'), ('f1', 'O')]), PhysicalType('time')), dtype=[('f0', 'O'), ('f1', 'O')])
         >>> su.si
         Unit("((1.49598e+11 m, 1.73146e+06 m / s), 3.15576e+07 s)")
 
     """
+
     def __new__(cls, units, names=None):
         dtype = None
         if names is not None:
@@ -127,7 +143,7 @@ class StructuredUnit:
                 names = names.field_names
             elif isinstance(names, np.dtype):
                 if not names.fields:
-                    raise ValueError('dtype should be structured, with fields.')
+                    raise ValueError("dtype should be structured, with fields.")
                 dtype = np.dtype([(name, DTYPE_OBJECT) for name in names.names])
                 names = _names_from_dtype(names)
             else:
@@ -150,7 +166,7 @@ class StructuredUnit:
                 units = (units,)
 
         if names is None:
-            names = tuple(f'f{i}' for i in range(len(units)))
+            names = tuple(f"f{i}" for i in range(len(units)))
 
         elif len(units) != len(names):
             raise ValueError("lengths of units and field names must match.")
@@ -166,15 +182,21 @@ class StructuredUnit:
                 # We are at the lowest level.  Check unit.
                 unit = Unit(unit)
                 if dtype is not None and isinstance(unit, StructuredUnit):
-                    raise ValueError("units do not match in depth with field "
-                                     "names from dtype or structured unit.")
+                    raise ValueError(
+                        "units do not match in depth with field "
+                        "names from dtype or structured unit."
+                    )
 
             converted.append(unit)
 
         self = super().__new__(cls)
         if dtype is None:
-            dtype = np.dtype([((name[0] if isinstance(name, list) else name),
-                               DTYPE_OBJECT) for name in names])
+            dtype = np.dtype(
+                [
+                    ((name[0] if isinstance(name, list) else name), DTYPE_OBJECT)
+                    for name in names
+                ]
+            )
         # Decay array to void so we can access by field name and number.
         self._units = np.array(tuple(converted), dtype)[()]
         return self
@@ -186,9 +208,10 @@ class StructuredUnit:
     @property
     def field_names(self):
         """Possibly nested tuple of the field names of the parts."""
-        return tuple(([name, unit.field_names]
-                      if isinstance(unit, StructuredUnit) else name)
-                     for name, unit in self.items())
+        return tuple(
+            ([name, unit.field_names] if isinstance(unit, StructuredUnit) else name)
+            for name, unit in self.items()
+        )
 
     # Allow StructuredUnit to be treated as an (ordered) mapping.
     def __len__(self):
@@ -223,8 +246,11 @@ class StructuredUnit:
             If given, should be a subclass of `~numpy.void`. By default,
             will return a new `~astropy.units.StructuredUnit` instance.
         """
-        results = np.array(tuple([func(part) for part in self.values()]),
-                           self._units.dtype)[()]
+        applied = tuple(func(part) for part in self.values())
+        if NUMPY_LT_1_24:
+            results = np.array(applied, self._units.dtype)[()]
+        else:
+            results = np.void(applied, self._units.dtype)
         if cls is not None:
             return results.view((cls, results.dtype))
 
@@ -239,7 +265,7 @@ class StructuredUnit:
         This is useful since ``np.array(value)`` would treat tuples as lower
         levels of the array, rather than as elements of a structured array.
         The routine does presume that the type of the first tuple is
-        representative of the rest.  Used in ``_get_converter``.
+        representative of the rest.  Used in ``get_converter``.
 
         For the special value of ``UNITY``, all fields are assumed to be 1.0,
         and hence this will return an all-float dtype.
@@ -256,13 +282,14 @@ class StructuredUnit:
         for (name, unit), part in zip(self.items(), value):
             if isinstance(unit, StructuredUnit):
                 descr.append(
-                    (name, unit._recursively_get_dtype(part, enter_lists=False)))
+                    (name, unit._recursively_get_dtype(part, enter_lists=False))
+                )
             else:
                 # Got a part associated with a regular unit. Gets its dtype.
                 # Like for Quantity, we cast integers to float.
                 part = np.array(part)
                 part_dtype = part.dtype
-                if part_dtype.kind in 'iu':
+                if part_dtype.kind in "iu":
                     part_dtype = np.dtype(float)
                 descr.append((name, part_dtype, part.shape))
         return np.dtype(descr)
@@ -270,25 +297,28 @@ class StructuredUnit:
     @property
     def si(self):
         """The `StructuredUnit` instance in SI units."""
-        return self._recursively_apply(operator.attrgetter('si'))
+        return self._recursively_apply(operator.attrgetter("si"))
 
     @property
     def cgs(self):
         """The `StructuredUnit` instance in cgs units."""
-        return self._recursively_apply(operator.attrgetter('cgs'))
+        return self._recursively_apply(operator.attrgetter("cgs"))
 
     # Needed to pass through Unit initializer, so might as well use it.
-    def _get_physical_type_id(self):
+    @cached_property
+    def _physical_type_id(self):
         return self._recursively_apply(
-            operator.methodcaller('_get_physical_type_id'), cls=Structure)
+            operator.attrgetter("_physical_type_id"), cls=Structure
+        )
 
     @property
     def physical_type(self):
         """Physical types of all the fields."""
         return self._recursively_apply(
-            operator.attrgetter('physical_type'), cls=Structure)
+            operator.attrgetter("physical_type"), cls=Structure
+        )
 
-    def decompose(self, bases=set()):
+    def decompose(self, bases: Collection[UnitBase] = ()) -> Self:
         """The `StructuredUnit` composed of only irreducible units.
 
         Parameters
@@ -305,8 +335,7 @@ class StructuredUnit:
         `~astropy.units.StructuredUnit`
             With the unit for each field containing only irreducible units.
         """
-        return self._recursively_apply(
-            operator.methodcaller('decompose', bases=bases))
+        return self._recursively_apply(operator.methodcaller("decompose", bases=bases))
 
     def is_equivalent(self, other, equivalencies=[]):
         """`True` if all fields are equivalent to the other's fields.
@@ -333,23 +362,22 @@ class StructuredUnit:
             return False
 
         for self_part, other_part in zip(self.values(), other.values()):
-            if not self_part.is_equivalent(other_part,
-                                           equivalencies=equivalencies):
+            if not self_part.is_equivalent(other_part, equivalencies=equivalencies):
                 return False
 
         return True
 
-    def _get_converter(self, other, equivalencies=[]):
+    def get_converter(self, other, equivalencies=[]):
         if not isinstance(other, type(self)):
             other = self.__class__(other, names=self)
 
-        converters = [self_part._get_converter(other_part,
-                                               equivalencies=equivalencies)
-                      for (self_part, other_part) in zip(self.values(),
-                                                         other.values())]
+        converters = [
+            self_part.get_converter(other_part, equivalencies=equivalencies)
+            for (self_part, other_part) in zip(self.values(), other.values())
+        ]
 
         def converter(value):
-            if not hasattr(value, 'dtype'):
+            if not hasattr(value, "dtype"):
                 value = np.array(value, self._recursively_get_dtype(value))
             result = np.empty_like(value)
             for name, converter_ in zip(result.dtype.names, converters):
@@ -359,6 +387,8 @@ class StructuredUnit:
 
         return converter
 
+    get_converter.__doc__ = UnitBase.get_converter.__doc__
+
     def to(self, other, value=np._NoValue, equivalencies=[]):
         """Return values converted to the specified unit.
 
@@ -394,17 +424,17 @@ class StructuredUnit:
             # We do not have UNITY as a default, since then the docstring
             # would list 1.0 as default, yet one could not pass that in.
             value = UNITY
-        return self._get_converter(other, equivalencies=equivalencies)(value)
+        return self.get_converter(other, equivalencies=equivalencies)(value)
 
-    def to_string(self, format='generic'):
+    def to_string(self, format="generic"):
         """Output the unit in the given format as a string.
 
         Units are separated by commas.
 
         Parameters
         ----------
-        format : `astropy.units.format.Base` instance or str
-            The name of a format or a formatter object.  If not
+        format : `astropy.units.format.Base` subclass or str
+            The name of a format or a formatter class.  If not
             provided, defaults to the generic format.
 
         Notes
@@ -414,22 +444,22 @@ class StructuredUnit:
 
         """
         parts = [part.to_string(format) for part in self.values()]
-        out_fmt = '({})' if len(self) > 1 else '({},)'
-        if format.startswith('latex'):
+        out_fmt = "({})" if len(self) > 1 else "({},)"
+        if format.startswith("latex"):
             # Strip $ from parts and add them on the outside.
             parts = [part[1:-1] for part in parts]
-            out_fmt = '$' + out_fmt + '$'
-        return out_fmt.format(', '.join(parts))
+            out_fmt = "$" + out_fmt + "$"
+        return out_fmt.format(", ".join(parts))
 
     def _repr_latex_(self):
-        return self.to_string('latex')
+        return self.to_string("latex")
 
     __array_ufunc__ = None
 
     def __mul__(self, other):
         if isinstance(other, str):
             try:
-                other = Unit(other, parse_strict='silent')
+                other = Unit(other, parse_strict="silent")
             except Exception:
                 return NotImplemented
         if isinstance(other, UnitBase):
@@ -441,6 +471,7 @@ class StructuredUnit:
         # Anything not like a unit, try initialising as a structured quantity.
         try:
             from .quantity import Quantity
+
             return Quantity(other, unit=self)
         except Exception:
             return NotImplemented
@@ -451,7 +482,7 @@ class StructuredUnit:
     def __truediv__(self, other):
         if isinstance(other, str):
             try:
-                other = Unit(other, parse_strict='silent')
+                other = Unit(other, parse_strict="silent")
             except Exception:
                 return NotImplemented
 
@@ -463,6 +494,7 @@ class StructuredUnit:
     def __rlshift__(self, m):
         try:
             from .quantity import Quantity
+
             return Quantity(m, self, copy=False, subok=True)
         except Exception:
             return NotImplemented
@@ -500,6 +532,7 @@ class Structure(np.void):
     `FutureWarning` about comparisons is given.
 
     """
+
     # Note that it is important for physical type IDs to not be stored in a
     # tuple, since then the physical types would be treated as alternatives in
     # :meth:`~astropy.units.UnitBase.is_equivalent`.  (Of course, in that
@@ -516,3 +549,36 @@ class Structure(np.void):
             other = other.item()
 
         return self.item() != other
+
+
+def _structured_unit_like_dtype(
+    unit: UnitBase | StructuredUnit, dtype: np.dtype
+) -> StructuredUnit:
+    """Make a `StructuredUnit` of one unit, with the structure of a `numpy.dtype`.
+
+    Parameters
+    ----------
+    unit : UnitBase
+        The unit that will be filled into the structure.
+    dtype : `numpy.dtype`
+        The structure for the StructuredUnit.
+
+    Returns
+    -------
+    StructuredUnit
+    """
+    if isinstance(unit, StructuredUnit):
+        # If unit is structured, it should match the dtype. This function is
+        # only used in Quantity, which performs this check, so it's fine to
+        # return as is.
+        return unit
+
+    # Make a structured unit
+    units = []
+    for name in dtype.names:
+        subdtype = dtype.fields[name][0]
+        if subdtype.names is not None:
+            units.append(_structured_unit_like_dtype(unit, subdtype))
+        else:
+            units.append(unit)
+    return StructuredUnit(tuple(units), names=dtype.names)
