@@ -9,24 +9,26 @@ import numpy as np
 from astropy.utils import minversion
 
 __all__ = [
-    "NUMPY_LT_1_22_1",
-    "NUMPY_LT_1_23",
+    "COPY_IF_NEEDED",
     "NUMPY_LT_1_24",
     "NUMPY_LT_1_25",
     "NUMPY_LT_1_26",
     "NUMPY_LT_2_0",
-    "COPY_IF_NEEDED",
+    "NUMPY_LT_2_1",
+    "NUMPY_LT_2_2",
+    "NUMPY_LT_2_3",
 ]
 
 # TODO: It might also be nice to have aliases to these named for specific
 # features/bugs we're checking for (ex:
 # astropy.table.table._BROKEN_UNICODE_TABLE_SORT)
-NUMPY_LT_1_22_1 = not minversion(np, "1.22.1")
-NUMPY_LT_1_23 = not minversion(np, "1.23")
 NUMPY_LT_1_24 = not minversion(np, "1.24")
 NUMPY_LT_1_25 = not minversion(np, "1.25")
 NUMPY_LT_1_26 = not minversion(np, "1.26")
-NUMPY_LT_2_0 = not minversion(np, "2.0.dev")
+NUMPY_LT_2_0 = not minversion(np, "2.0")
+NUMPY_LT_2_1 = not minversion(np, "2.1.0.dev")
+NUMPY_LT_2_2 = not minversion(np, "2.2.0.dev0")
+NUMPY_LT_2_3 = not minversion(np, "2.3.0.dev0")
 
 
 COPY_IF_NEEDED = False if NUMPY_LT_2_0 else None
