@@ -1,14 +1,16 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """Sundry function and class decorators."""
 
-
 import functools
 import inspect
+import sys
 import textwrap
 import threading
 import types
 import warnings
+from functools import wraps
 from inspect import signature
+from types import FunctionType
 
 from .exceptions import (
     AstropyDeprecationWarning,
@@ -37,6 +39,8 @@ def deprecated(
     pending=False,
     obj_type=None,
     warning_type=AstropyDeprecationWarning,
+    *,
+    pending_warning_type=AstropyPendingDeprecationWarning,
 ):
     """
     Used to mark a function or class as deprecated.
@@ -46,8 +50,8 @@ def deprecated(
     Parameters
     ----------
     since : str
-        The release at which this API became deprecated.  This is
-        required.
+        The release at which this API became deprecated.
+        This is required.
 
     message : str, optional
         Override the default deprecation message.  The format
@@ -74,7 +78,7 @@ def deprecated(
         tell the user about this alternative if provided.
 
     pending : bool, optional
-        If True, uses a AstropyPendingDeprecationWarning instead of a
+        If True, uses a ``pending_warning_type`` instead of a
         ``warning_type``.
 
     obj_type : str, optional
@@ -84,6 +88,11 @@ def deprecated(
     warning_type : Warning
         Warning to be issued.
         Default is `~astropy.utils.exceptions.AstropyDeprecationWarning`.
+
+    pending_warning_type : Warning
+        Pending warning to be issued.
+        This only works if ``pending`` is set to True.
+        Default is `~astropy.utils.exceptions.AstropyPendingDeprecationWarning`.
     """
     method_types = (classmethod, staticmethod, types.MethodType)
 
@@ -125,7 +134,7 @@ def deprecated(
 
         def deprecated_func(*args, **kwargs):
             if pending:
-                category = AstropyPendingDeprecationWarning
+                category = pending_warning_type
             else:
                 category = warning_type
 
@@ -236,6 +245,7 @@ def deprecated_attribute(
     alternative=None,
     pending=False,
     warning_type=AstropyDeprecationWarning,
+    pending_warning_type=AstropyPendingDeprecationWarning,
 ):
     """
     Used to mark a public attribute as deprecated.  This creates a
@@ -273,6 +283,11 @@ def deprecated_attribute(
         Warning to be issued.
         Default is `~astropy.utils.exceptions.AstropyDeprecationWarning`.
 
+    pending_warning_type : Warning
+        Pending warning to be issued.
+        This only works if ``pending`` is set to True.
+        Default is `~astropy.utils.exceptions.AstropyPendingDeprecationWarning`.
+
     Examples
     --------
     ::
@@ -302,6 +317,7 @@ def deprecated_attribute(
         alternative=alternative,
         pending=pending,
         warning_type=warning_type,
+        pending_warning_type=pending_warning_type,
     )
 
     @specific_deprecated
@@ -608,6 +624,102 @@ def deprecated_renamed_argument(
     return decorator
 
 
+def future_keyword_only(names: list[str], *, since=list[str]) -> FunctionType:
+    """Decorator to mark one or more function parameter(s) as future keyword-only.
+
+    Parameters
+    ----------
+    names: list[str]
+        names of parameters to be marked as future keyword-only
+    since: list[str]
+        versions in which each parameter was marked
+
+    Examples
+    --------
+    >>> @future_keyword_only(['spam', 'eggs'], since=['7.1', '7.1'])
+    ... def bacon(spam, eggs):
+    ...     ...
+    """
+
+    def decorate(raw_function):
+        # validate the conditions in which the decorator is used
+        # this adds a small overhead on startup time but not on function call
+
+        future_kwo_names = names
+        if len(future_kwo_names) != len(since):
+            raise ValueError(
+                "Expected names and since values with "
+                "identical length. "
+                f"Got {len(names)=} and {len(since)=}"
+            )
+        params = signature(raw_function).parameters
+        params_names: list[str] = list(params)
+        future_kwo_to_since = dict(zip(future_kwo_names, since))
+
+        # check that all names exist in the current signature
+        if any(n not in params for n in future_kwo_names):
+            unknown = sorted(set(future_kwo_names) - set(params_names))
+            raise ValueError(
+                "The following arguments cannot be marked as future keyword-only "
+                "because they were not found in the decorated function's signature: "
+                f"{', '.join(unknown)}"
+            )
+
+        # check that every future kwo is currently allowed as positional-or-keyword
+        # (otherwise something's wrong with the decorator call itself)
+        pos_or_kw_names = {
+            n for (n, p) in params.items() if p.kind is p.POSITIONAL_OR_KEYWORD
+        }
+        if not pos_or_kw_names.issuperset(future_kwo_names):
+            diff = sorted(set(future_kwo_names) - pos_or_kw_names)
+            raise ValueError(
+                "The following arguments cannot be marked as future keyword-only "
+                "because they are not currently positional-or-keyword: "
+                f"{', '.join(diff)}"
+            )
+
+        # check that there are no other positionally allowed arguments beyond
+        # any future kwo (otherwise these arguments could still break)
+        future_kwo_positions = [params_names.index(n) for n in future_kwo_names]
+        idx_min = min(future_kwo_positions)
+        idx_max = max(future_kwo_positions) + 1
+        while idx_max < len(params):
+            p = params[params_names[idx_max]]
+            if p.kind in (p.KEYWORD_ONLY, p.VAR_KEYWORD):
+                break
+            idx_max += 1
+        if any(n not in future_kwo_names for n in list(params)[idx_min:idx_max]):
+            broken = params_names[max(future_kwo_positions) + 1 : idx_max]
+            raise ValueError(
+                "The following positionally-allowed arguments were not marked as "
+                "future keyword-only and would be broken under future keyword-only "
+                f"requirements: {', '.join(broken)}"
+            )
+
+        @wraps(raw_function)
+        def decorated_function(*args, **kwargs):
+            if len(args) > idx_min:
+                flagged_positions = range(idx_min, len(args))
+                flagged_names = [params_names[pos] for pos in flagged_positions]
+                depr_versions = [future_kwo_to_since[n] for n in flagged_names]
+                if len(set(depr_versions)) == 1:
+                    since_details = depr_versions[0]
+                else:
+                    since_details = ", ".join(depr_versions) + ", respectively"
+                msg = (
+                    "The following arguments were received positionally, which "
+                    f"will be disallowed in a future release: {', '.join(flagged_names)}\n"
+                    "Pass them as keywords to suppress this warning. "
+                    f"(deprecated since {since_details})"
+                )
+                warnings.warn(msg, AstropyDeprecationWarning, stacklevel=2)
+            return raw_function(*args, **kwargs)
+
+        return decorated_function
+
+    return decorate
+
+
 # TODO: This can still be made to work for setters by implementing an
 # accompanying metaclass that supports it; we just don't need that right this
 # second
@@ -733,7 +845,7 @@ class classproperty(property):
         # the doc argument was used rather than taking the docstring
         # from fget
         # Related Python issue: https://bugs.python.org/issue24766
-        if doc is not None:
+        if doc is not None and sys.flags.optimize < 2:
             self.__doc__ = doc
 
     def __get__(self, obj, objtype):
@@ -1120,6 +1232,9 @@ def format_doc(docstring, *args, **kwargs):
     on an object to first parse the new docstring and then to parse the
     original docstring or the ``args`` and ``kwargs``.
     """
+    if sys.flags.optimize >= 2:
+        # docstrings are dropped at runtime, so let's return a noop decorator
+        return lambda func: func
 
     def set_docstring(obj):
         if docstring is None:
@@ -1142,6 +1257,13 @@ def format_doc(docstring, *args, **kwargs):
                 "docstring that is not empty."
             )
 
+        # Dedent both the original and the new docstring to ensure consistent
+        # leading whitespace, because from Python 3.13 the bytecode compiler
+        # strips leading whitespace from docstrings. If the text in ``doc``
+        # has any leading whitespace, this can lead to reST/Sphinx errors.
+        if sys.version_info[:2] >= (3, 13):
+            doc = textwrap.dedent(doc).lstrip("\n")
+
         # If the original has a not-empty docstring append it to the format
         # kwargs.
         kwargs["__doc__"] = obj.__doc__ or ""
