@@ -3,19 +3,18 @@
 import numpy as np
 import pytest
 from numpy import ma
-from numpy.testing import assert_allclose, assert_equal
-from packaging.version import Version
+from numpy.testing import assert_allclose, assert_array_equal, assert_equal
 
-from astropy.utils.compat.optional_deps import HAS_MATPLOTLIB, HAS_PLT
+from astropy.utils.compat.optional_deps import HAS_PLT
 from astropy.visualization.interval import ManualInterval, PercentileInterval
-from astropy.visualization.mpl_normalize import ImageNormalize, imshow_norm, simple_norm
+from astropy.visualization.mpl_normalize import (
+    ImageNormalize,
+    SimpleNorm,
+    imshow_norm,
+    simple_norm,
+)
 from astropy.visualization.stretch import LogStretch, PowerStretch, SqrtStretch
 
-if HAS_MATPLOTLIB:
-    import matplotlib
-
-    MATPLOTLIB_LT_32 = Version(matplotlib.__version__) < Version("3.2")
-
 DATA = np.linspace(0.0, 15.0, 6)
 DATA2 = np.arange(3)
 DATA2SCL = 0.5 * DATA2
@@ -24,7 +23,7 @@ STRETCHES = (SqrtStretch(), PowerStretch(0.5), LogStretch())
 INVALID = (None, -np.inf, -1)
 
 
-@pytest.mark.skipif(HAS_MATPLOTLIB, reason="matplotlib is installed")
+@pytest.mark.skipif(HAS_PLT, reason="matplotlib is installed")
 def test_normalize_error_message():
     with pytest.raises(
         ImportError, match=r"matplotlib is required in order to use this class."
@@ -32,12 +31,17 @@ def test_normalize_error_message():
         ImageNormalize()
 
 
-@pytest.mark.skipif(not HAS_MATPLOTLIB, reason="requires matplotlib")
+@pytest.mark.skipif(not HAS_PLT, reason="requires matplotlib")
 class TestNormalize:
     def test_invalid_interval(self):
         with pytest.raises(TypeError):
             ImageNormalize(vmin=2.0, vmax=10.0, interval=ManualInterval, clip=True)
 
+    def test_invalid_vmin_vmax(self):
+        with pytest.raises(ValueError):
+            norm = ImageNormalize(vmin=10.0, vmax=2.0)
+            norm(10)
+
     def test_invalid_stretch(self):
         with pytest.raises(TypeError):
             ImageNormalize(vmin=2.0, vmax=10.0, stretch=SqrtStretch, clip=True)
@@ -54,6 +58,11 @@ class TestNormalize:
         assert_allclose(norm(6), 0.70710678)
         assert_allclose(norm(6), norm2(6))
 
+    def test_vmin_vmax_equal(self):
+        norm = ImageNormalize(vmin=2.0, vmax=2.0)
+        data = np.arange(10) - 5.0
+        assert_array_equal(norm(data), 0)
+
     def test_clip(self):
         norm = ImageNormalize(vmin=2.0, vmax=10.0, stretch=SqrtStretch(), clip=True)
         norm2 = ImageNormalize(
@@ -194,7 +203,7 @@ class TestNormalize:
         assert_equal(result2, result3)
 
 
-@pytest.mark.skipif(not HAS_MATPLOTLIB, reason="requires matplotlib")
+@pytest.mark.skipif(not HAS_PLT, reason="requires matplotlib")
 class TestImageScaling:
     def test_linear(self):
         """Test linear scaling."""
@@ -210,7 +219,7 @@ class TestImageScaling:
     def test_sqrt_invalid_kw(self, invalid):
         stretch = SqrtStretch()
         norm1 = simple_norm(
-            DATA3, stretch="sqrt", min_cut=-1, max_cut=1, clip=False, invalid=invalid
+            DATA3, stretch="sqrt", vmin=-1, vmax=1, clip=False, invalid=invalid
         )
         norm2 = ImageNormalize(
             stretch=stretch, vmin=-1, vmax=1, clip=False, invalid=invalid
@@ -258,7 +267,7 @@ class TestImageScaling:
 
     def test_min(self):
         """Test linear scaling."""
-        norm = simple_norm(DATA2, stretch="linear", min_cut=1.0, clip=True)
+        norm = simple_norm(DATA2, stretch="linear", vmin=1.0, clip=True)
         assert_allclose(norm(DATA2), [0.0, 0.0, 1.0], atol=0, rtol=1.0e-5)
 
     def test_percent(self):
@@ -277,31 +286,57 @@ class TestImageScaling:
             simple_norm(DATA2, stretch="invalid")
 
 
-@pytest.mark.skipif(not HAS_PLT, reason="requires matplotlib.pyplot")
+@pytest.mark.skipif(not HAS_PLT, reason="requires matplotlib")
+@pytest.mark.parametrize("stretch", ["linear", "sqrt", "power", "log", "asinh", "sinh"])
+def test_simplenorm(stretch):
+    data = np.arange(25).reshape((5, 5))
+    snorm = SimpleNorm(stretch, percent=99)
+    norm = snorm(data)
+    assert isinstance(norm, ImageNormalize)
+    assert_allclose(norm(data), simple_norm(data, stretch, percent=99)(data))
+
+
+@pytest.mark.skipif(not HAS_PLT, reason="requires matplotlib")
+def test_simplenorm_imshow():
+    from matplotlib.figure import Figure
+    from matplotlib.image import AxesImage
+
+    data = np.arange(25).reshape((5, 5))
+    fig = Figure()
+    ax = fig.add_subplot()
+    snorm = SimpleNorm("sqrt", percent=99)
+    axim = snorm.imshow(data, ax=ax)
+    assert isinstance(axim, AxesImage)
+    keys = ("vmin", "vmax", "stretch", "clip", "invalid")
+    for key in keys:
+        assert getattr(axim.norm, key) == getattr(snorm(data), key)
+
+    fig.clear()
+    axim = snorm.imshow(data, ax=None)
+
+    with pytest.raises(ValueError):
+        snorm.imshow(data, ax=ax, norm=ImageNormalize())
+
+
+@pytest.mark.skipif(not HAS_PLT, reason="requires matplotlib")
 def test_imshow_norm():
-    import matplotlib.pyplot as plt
+    from matplotlib.figure import Figure
 
     image = np.random.randn(10, 10)
 
-    plt.clf()
-    ax = plt.subplot(label="test_imshow_norm")
+    fig = Figure()
+    ax = fig.add_subplot(label="test_imshow_norm")
     imshow_norm(image, ax=ax)
 
-    with pytest.raises(ValueError):
-        # X and data are the same, can't give both
-        imshow_norm(image, X=image, ax=ax)
-
     with pytest.raises(ValueError):
         # illegal to manually pass in normalization since that defeats the point
         imshow_norm(image, ax=ax, norm=ImageNormalize())
 
-    plt.clf()
+    fig.clear()
     imshow_norm(image, ax=ax, vmin=0, vmax=1)
 
-    # make sure the pyplot version works
-    plt.clf()
+    # make sure the matplotlib version works
+    fig.clear()
     imres, norm = imshow_norm(image, ax=None)
 
     assert isinstance(norm, ImageNormalize)
-
-    plt.close("all")
